/*** 
 * \file gre/3dtinter.h
 * \brief Base class for terrain decimation algorithms
 *
 * \if NODOC
 * $Id: 3dtinter.h_v 1.17 2013/02/15 20:54:10 mju Exp $
 *
 * $Log: 3dtinter.h_v $
 * Revision 1.17  2013/02/15 20:54:10  mju
 * Fix dox hdr.
 *
 * Revision 1.16  2011/10/06 15:21:34  mju
 * Remove dependencies.
 *
 * Revision 1.15  2005/10/27 13:07:58  mju
 * Use gre namespace instead of legacy types.
 *
 * Revision 1.3  2003/04/09 19:10:19  vdronov
 * terrain null value
 *
 * Revision 1.1  2003/03/10 15:08:24  vdronov
 * Initial revision
 * \endif
 */ 

#ifndef	INC_GRE_3DTINTER_H
#define	INC_GRE_3DTINTER_H

#ifndef	INC_GRE_3DTERRA_H
#include <gre/3dterra.h>
#endif

#ifndef	INC_RVC_TIN_H
#include <rvc/tin.h>
#endif

namespace GRE {

//! TIN terrain model.
class TINTERRAIN : public TERRAIN {
	public:

		TINTERRAIN (											//! Constructor 
			);

		~TINTERRAIN (											//! Destructor 
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DRECT3D ALIGN16(m_Extents);
		SPATREF::COORDREFSYS m_CoordRefSys;
		RVC::TIN m_TIN;
		TRIANGULATION m_Triangulation;
		BITSET m_TrianglesBits;				  

		TINTERRAIN& operator= (const TINTERRAIN&);

		TERRAIN* v_CreateInstance (
			) { return new TINTERRAIN(); };

		ERRVALUE v_Build (
			const GRE::LAYER *layer
			);

		bool v_IsObjectTypeSupported (
			const RVC::OBJTYPE objtype
			) const { return (objtype == RVC::OBJTYPE_TIN); };

		void v_Free (
			);

		ERRVALUE Create (					 
			const GRE::LAYER *layer
			);

		void Delete (					 
			);

		ERRVALUE v_GetDefaultScene ( 
			SCENE3D& scene
			) const;

		ERRVALUE v_SetScene (
			const SCENE3D& scene,
			const SIMPLE_ARRAY<TEXTURE*> &textures	  
			);

		ERRVALUE v_DrawScene (
			CONTROLLER &controller,			 
			DRAWINGCONTEXT3D& drawing,
			TEXTURE* texture,
			TEXTUREFILTER* texturefilter
			);

		ERRVALUE v_SetScene (
			const SCENE3D& scene,
			const SIMPLE_ARRAY<GEOMETRIC3D*> &geometrics,
			const bool wireframe
			);

		float v_GetElevation (
			const float x,
			const float y
			) const;

		ERRVALUE v_ComputeProfile (
			const POLYLINE& polyline,
			POLYLINE& profile,
			SIMPLE_ARRAY<INT32>& index
			);

		float v_GetAngleToNorth (
			const float x,
			const float y
			) const;

		bool v_HasNull (
			) const { return false; };

		bool v_IsNull (
			const float x,
			const float y
			) const { return false; };

		void v_SetCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys
			);

		const SPATREF::COORDREFSYS v_GetCoordRefSys (
			) const { return m_CoordRefSys; };

		const DRECT3D& v_GetExtents (
			) { return m_Extents; };

		void v_CreateBoundary (
			REGION2D& boundary
			);

		#endif //!< GENERATING_DOXYGEN_OUTPUT
	};

};		//! End of namespace

#endif

