/**
 * \file gre/dlgviewstereo2d.h
 * \brief Dialog to adjust stereo view settings
 *
 * \if NODOC
 * $Id: dlgviewstereo2d.h_v 1.9 2012/04/10 21:08:48 mju Exp $
 *
 * $Log: dlgviewstereo2d.h_v $
 * Revision 1.9  2012/04/10 21:08:48  mju
 * Add controls for "no terrain" color.
 *
 * Revision 1.8  2012/04/06 14:31:38  mju
 * Rename member.
 *
 * Revision 1.7  2012/04/04 20:48:15  mju
 * Add controls for shift view and showing mode indicator.
 *
 * Revision 1.6  2012/03/29 21:45:53  mju
 * Add getStereoRender.
 *
 * Revision 1.5  2012/03/23 17:41:02  mju
 * Remove toggle for auto-open.
 *
 * Revision 1.4  2012/02/03 18:29:21  mju
 * Add controls for how to get terrain if none.
 *
 * Revision 1.3  2008/03/25 15:55:18  mju
 * Add applyToGroup.
 *
 * Revision 1.2  2007/05/07 17:55:18  mju
 * Add method.
 *
 * Revision 1.1  2007/05/04 21:43:19  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_DLGVIEWSTEREO2D_H
#define	INC_GRE_DLGVIEWSTEREO2D_H

#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef INC_MGUI_FORMRADIOBUTTONS_H
#include <mgui/formradiobuttons.h>
#endif

#ifndef INC_MGUI_COLORBTN_H
#include <mgui/colorbtn.h>
#endif

#ifndef INC_MGUI_SLIDER_H
#include <mgui/slider.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class STEREORENDER;
namespace GRE {
	class GROUP;
	namespace UI {
		class FORM_VIEW;
		}
	}
#endif


namespace GRE {

class DLG_VIEWSTEREO2D : public MGUI::DLGSHELL {
	public:
		DLG_VIEWSTEREO2D (
			GRE::UI::FORM_VIEW *pFormView
			);
		~DLG_VIEWSTEREO2D (
			);
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_VIEWSTEREO2D THISCLASS;
		GRE::UI::FORM_VIEW *m_pFormView;
		MGUI::FORM_EDIT_NUMBER m_EditRelativeDepthScale;
		MGUI::FORM_EDIT_NUMBER m_EditMaxReliefExaggeration;
		MGUI::FORM_EDIT_NUMBER m_EditPctDepthBeyondScreen;
		MGUI::FORM_COMBOBOX m_CbxShiftView;
		MGUI::CTRL_TOGGLEBUTTON m_TglShowStereoViewIndicators;
		MGUI::FORM_COLORBUTTON m_CbnNoTerrainColor;
		MGUI::FORM_SLIDER m_CtlNoTerrainColorAmt;
		MGUI::CTRL_GROUPBOX m_GbxActionWhenNoTerrain;
		MGUI::CTRL_LABEL m_LblActionWhenNoTerrain;
		MGUI::FORM_RADIOBUTTONS m_RbnActionWhenNoTerrain;
		MGUI::CTRL_PUSHBUTTON m_BtnDeviceSettings;
		bool m_RenderSettingsChanged;

		void ApplyToGroup (GRE::GROUP *group);
		void GetStereoRender (STEREORENDER& StereoRender);
		void OnChangeSetting ();
		void OnCmdDeviceSettings ();
		void OnRbnActionWhenNoTerrain ();
		void UpdateTitle ();

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnApply ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}

#endif
