/**
 * \file gre/lrarcims.h
 * \brief GRE LAYER_ARCIMS definitions
 *
 * \if NODOC
 * $Id: lrarcims.h_v 1.11 2011/10/20 12:49:30 mju Exp $
 *
 * $Log: lrarcims.h_v $
 * Revision 1.11  2011/10/20 12:49:30  mju
 * Remove redundant hdr.
 *
 * Revision 1.10  2010/11/19 23:00:41  bcolombe
 * Remove use of DLG_GETOBJECT::ITEM
 *
 * Revision 1.9  2010/08/19 23:01:55  bcolombe
 * Add v_AssignItem
 *
 * Revision 1.8  2010/08/16 13:22:46  mju
 * Only use dlg_getobjectitem hdr if use_mgui.
 *
 * Revision 1.6  2010/08/11 21:54:10  bcolombe
 * Added ability to contruct layer from DLG_GETOBJECT::ITEM
 *
 * Revision 1.5  2008/07/16 23:04:02  bcolombe
 * Added SetupLegend and BuildGetLegendRequest
 *
 * Revision 1.4  2008/04/09 16:37:28  mju
 * Add v_treeViewAddUpdateLegend.
 *
 * Revision 1.3  2007/10/17 15:13:03  dwilliss
 * Added a parameter to a private method
 *
 * Revision 1.2  2007/09/21 15:42:35  dwilliss
 * Added legend support
 *
 * Revision 1.1  2007/09/14 21:45:46  dwilliss
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LRARCIMS_H
#define	INC_GRE_LRARCIMS_H

#ifndef	INC_GRE_LRIMAGE_H
	#include <gre/lrimage.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
	#include <mi32/mistringlist.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace SOA {
	namespace ARCIMS {
		class CONNECTION;
		}
	}
namespace SPATMOD {
	namespace IMAGE {
		class PIXEL;
		class PIXELTABLE;
		class STAGE_AUTOPTR;
		}
	}
class HTTPCLIENT;
class HTTPMESSAGE;
class XMLDOC;
#endif

namespace GRE {
//===================================================================================================================

//! Raster Layer class
class LAYER_ARCIMS : public GRE::LAYER_IMAGE {
	public:

		// LAYER_ARCIMS::DISPPARM
		class DISPPARM : public LAYER::DISPPARM {
			public:

				DISPPARM ();

				DISPPARM (
					const DISPPARM& rhs
					);

				virtual ~DISPPARM ();

				DISPPARM& operator= (
					const DISPPARM& rhs
					) { if (this != &rhs) { Copy(rhs); } return (*this); }

				//! Build request URL for "GetMap".
				ERRVALUE BuildGetImageRequest (
					const LAYERDC& layerdc,
					HTTPMESSAGE& RequestHdrs,
					XMLDOC& RequestBody,
					LPOINT2D& TgtSize,
					TRANS2D_MAPGEN& TransToCRS
					) const;

				ERRVALUE BuildGetLegendRequest (
					HTTPMESSAGE& RequestHdrs,
					XMLDOC& RequestBody
					) const;

				//! Build pixel table for detecting transparent pixels.
				ERRVALUE BuildPixelTableTransparent (
					const COLOR& ColorBG,
					SPATMOD::IMAGE::PIXELTABLE& PixelTable
					) const;

				//! Compute preference for format to use in GetMap request.
				//! @return Preference rating, 0 indicates not supported.
				static int ComputeFormatGetMapPreference (
					const MISTRING& Format
					);

				//! Compute region for current settings.
				ERRVALUE ComputeRegion (
					REGION2D& RegionUsed
					) const;

				//! Get connection to ARCIMS.
				const SOA::ARCIMS::CONNECTION& GetConnection (
					) const { return (*m_pConnection); }

				//! Get coordinate reference system.
				const SPATREF::COORDREFSYS& GetCoordRefSys (
					) const { return (m_CoordRefSys); }

				//! Get background pixel detection setting.
				bool GetDetectBackground (
					) const { return (m_DetectBackground); }

				//! Get black pixel detection setting.
				bool GetDetectBlack (
					) const { return (m_DetectBlack); }

				//! Get white pixel detection setting.
				bool GetDetectWhite (
					) const { return (m_DetectWhite); }

				//! Get format for GetMap request.
				const MISTRING& GetFormatGetMap (
					) const { return (m_FormatGetMap); }

				//! Get layer name list.
				const MISTRINGLIST& GetLayerNameList (
					) const { return (m_LayerNameList); }

				//! Get URL string for server.
				const MISTRING& GetServerURL (
					) const;

				//! Get transparency (alpha) value.
				UINT8 GetTransparency (
					) const { return (m_Transparency); }

				//! Determine if has been validated.
				bool IsValidated (
					) const { return (m_IsValidated); }

				//! Set connection to ARCIMS.
				void SetConnection (
					const SOA::ARCIMS::CONNECTION& connection
					);

				//! Set coordinate reference system.
				void SetCoordRefSys (
					const SPATREF::COORDREFSYS& CoordRefSys
					);

				//! Set whether to detect background pixels if format has no alpha component.
				void SetDetectBackground (
					bool DetectBackground
					) { m_DetectBackground = DetectBackground; }

				//! Set whether to detect black pixels an treat as transparent.
				void SetDetectBlack (
					bool DetectBlack
					) { m_DetectBlack = DetectBlack; }

				//! Set whether to detect white pixels an treat as transparent.
				void SetDetectWhite (
					bool DetectWhite
					) { m_DetectWhite = DetectWhite; }

				//! Set to default format for GetMap request.
				void SetDftFormatGetMap (
					);

				//! Set to default layer for current service.
				void SetDftLayer (
					);

				//! Set format for GetMap request.
				void SetFormatGetMap (
					const MISTRING& Format
					);

				//! Set layer list (names and styles).
				ERRVALUE SetLayerList (
					const MISTRINGLIST& LayerNameList
					);

				//! Get transparency (alpha) value.
				void SetTransparency (
					UINT8 Transparency
					) { m_Transparency = Transparency; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				SOA::ARCIMS::CONNECTION *m_pConnection;
				MISTRINGLIST m_LayerNameList;
				SPATREF::COORDREFSYS m_CoordRefSys;
				MISTRING m_FormatGetMap;
				UINT8 m_Transparency;
				bool m_DetectBackground;
				bool m_DetectBlack;
				bool m_DetectWhite;
				bool m_IsValidated;


				void Copy (const DISPPARM&);
				static ERRVALUE SerializerCB_Connection (SERIALIZER& serializer, const SERIALIZERITEM *item, void *vDispParm, SERIALIZERITEM::ACTION action);

				// SERIALIZABLE overrides
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;

				// LAYER::DISPPARM overrides
				virtual ERRVALUE v_Validate ();

				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_ARCIMS::DISPPARM

		//! Register this layer type
		static void RegisterType (
			);

		//! Method to install SML class, called by internal SML function.
		static void SMLClassInstall (
			SMLCONTEXT *context							//!< SML context
			);

		//! Construct layer and insert into list
		explicit LAYER_ARCIMS (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		virtual ~LAYER_ARCIMS ();

		//! Retrieve layer-specific display parameters.
		//! @return current parameters
		const DISPPARM& GetDispParm (
			) const {
			return (m_dispparm);
			}

		//! Set layer-specific display parameters.
		//!
		//! Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef LAYER_IMAGE BASECLASS;
		class DLG;

	#ifdef USE_MGUI
		DLG *m_pDlgControlPanel;
	#endif
		LEGEND_IMAGESAMPLE m_ImageSampleLegend;

		static void LegendMsgHandler (GRAPHICTREEITEM_MSG *msg);

	#ifdef USE_MGUI
		void DestroyDlgControlPanel ();
	#endif
		ERRVALUE Draw2D (LAYERDC& layerdc);
		void SetupLegend ();
		ERRVALUE GetImageFromReply (
			const XMLDOC& xmldoc,
			HTTPCLIENT& Client,
			const char* element,	//!< Either "OUTPUT" or "LEGEND"
			SPATMOD::IMAGE::STAGE_AUTOPTR& StagePtr,
			TRANS2D_MAPGEN* Trans = 0
			);

		// Overrides from LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& item, MDLGPARENT dlgparent);
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent, UINT32 flags = 0);
	#endif
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		virtual ERRVALUE v_Draw (LAYERDC& layerdc);
		virtual bool v_HasNullImageCells () const;
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetDftName (bool ReplaceExisting);
	#ifdef USE_MGUI
		virtual MGUI::CTRL_TREE::ITEMHND v_TreeViewAddUpdateLegend (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
	#endif
		virtual ERRVALUE v_UpdateExtents (bool forceupdate);
		virtual const GRE::LEGEND* v_GetLegend (ELEMTYPE);
		virtual bool v_HasLegend () const;

		LAYER_ARCIMS (const LAYER_ARCIMS&);
		LAYER_ARCIMS& operator= (const LAYER_ARCIMS&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of LAYER_ARCIMS


//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRARCIMS_H
