/**
 * \file gre/lrlegnd2.h
 * \brief GRE::LAYER_LEGEND2 class definitions
 *
 * \if NODOC
 * $Id: lrlegnd2.h_v 1.13 2013/02/15 20:45:42 mju Exp $
 *
 * $Log: lrlegnd2.h_v $
 * Revision 1.13  2013/02/15 20:45:42  mju
 * Fix dox hdr.
 *
 * Revision 1.12  2013/01/18 15:56:07  mju
 * Make graphiclayout pointer so can forward-declare and remove hdrs.
 *
 * Revision 1.11  2006/09/15 15:30:28  mju
 * SErializer changes.
 *
 * Revision 1.1  2002/09/20 17:37:36  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRLEGND2_H
#define	INC_GRE_LRLEGND2_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
class GRAPHICLAYOUT;
#endif

namespace GRE {

//===================================================================================================================

class LAYER_LEGEND2 : public GRE::LAYER {
	public:

		//! Class used internally to keep track of which layers the legend refers to
		class LAYERREF  {
			public:
				MIUNICODE *m_name;	//!< Can't use MISTRING.  We MmAllocC() an array of LAYRREFs.
				//! Our DISPPARM's destructor worrys about freeing stuff.
				GRE::LAYER* m_layer;
				ELEMTYPE m_ElemType;
				UINT32 m_ID;

				LAYERREF (
					) :
					m_name(0),
					m_layer(0),
					m_ID(0),
					m_ElemType(ELEMTYPE_Empty)
					{
					}

				LAYERREF (
					GRE::LAYER* lyr,
					ELEMTYPE et
					);

				LAYERREF(const LAYERREF& rhs) :
					m_ID(rhs.m_ID),
					m_layer(rhs.m_layer),
					m_ElemType(rhs.m_ElemType) 
					{
						m_name = rhs.m_name ? ucstrdup(rhs.m_name) : 0;
					}

				~LAYERREF() {
					MmFree(m_name);
					}

				LAYERREF& operator= (const LAYERREF& rhs) {
					if (this != &rhs) {
						m_name = rhs.m_name ? ucstrdup(rhs.m_name) : 0;
						m_ID = rhs.m_ID;
						m_layer = rhs.m_layer;
						m_ElemType = rhs.m_ElemType;
						}
					return (*this);
					}

				//! Also sets m_name
				void SetLayer (
					GRE::LAYER* layer
					);

			};
			
		// LAYER_LEGEND2::DISPPARM
		class DISPPARM : public GRE::LAYER::DISPPARM {
			public:

				class DLG;

			/*=================This stuff must be saved/loaded by mdlegend.c=============*/
				GRAPHICLAYOUT *m_pGraphicLayout;
				UINT32 m_NumLayers;
				LAYERREF *m_layers;

			/*=================This stuff is just internal=========================*/

				DISPPARM ();
				DISPPARM (const DISPPARM& rhs) {
					Copy(rhs);
					}
				virtual ~DISPPARM (
					) { Free(); }

				DISPPARM& operator= (const DISPPARM& rhs) {
					if (this != &rhs) {
						Free();
						Copy(rhs);
						}
					return (*this);
					}

				//! Returns index into m_layers array for given layer/elemtype
				//! Return -1 if not found, < -1 if error.
				INT32 GetLayerRefNum (
					GRE::LAYER* layer,
					ELEMTYPE ElemType,
					bool bAllocIfNotFound = false
					);

				void OnLayerChange (
					INT32 LayerRefNum,
					bool deleting
					);

				void OnLayoutLoaded (
					GRE::VIEWABLE* viewable
					);

			protected:

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				LAYER_LEGEND2 *m_layer;		// Need it in a callback that only gets the DISPPARM

				static void GRECB_LayerCallback (
					const GRE::MSG* msg,
					void* cbdata
					);

				static int SerializerCB_GRLayout (
					SERIALIZER& serializer,
					const SERIALIZERITEM *itemdef,
					void* cbdata,
					SERIALIZERITEM::ACTION action
					);

				void Copy (const DISPPARM&);
				void Free ();
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;

				friend class LAYER_LEGEND2;
				#endif //!< GENERATING_DOXYGEN_OUTPUT
			};

		//! Callback for the "Add Legend" menu of GRE::LAYER_LEGEND.
	#ifndef WIN32_NATIVE
		static void CB_NewLegend2 (
			Widget w,
			void *vsubtype,
			void *
			);
	#endif

		//! Register this layer type
		static void RegisterType ();

		//! Construct layer and insert into list
		explicit LAYER_LEGEND2 (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			GRE::LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		virtual ~LAYER_LEGEND2 ();

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm (
			) const { return (m_dispparm); }

		//! Return scale factor between legend's layout units and output "ground meters"
		double GetScaleFactor (
			) const;

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	#if defined(X_NATIVE) && !defined(GENERATING_DOXYGEN_OUTPUT)
		static void CB_CreateDialog (Widget, void*, void*);
	#endif

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		static void GRECB_LayoutLoaded (const GRE::MSG* msg, void* cbdata);

		// Overrides from GRE::LAYER.
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual GRE::LAYER* v_Copy (GRE::GROUP *group) const;
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual ERRVALUE v_TemplateRead (SERIALIZER& serializer);
		virtual int v_UpdateExtents (bool forceupdate);
		virtual bool v_UsesTransparency (const MGD::DEVICE *pDevice = 0) const;

		friend class LAYER_LEGEND2::DISPPARM;	//!< Need access to private static GRECB_LayoutLoaded()

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_LRLEGND2_H
