/*
 * gre/uishape - GRE User Interface definitions for Shape layer
 *
 * \if NODOC
 * $Id: uishape.h_v 1.11 2011/10/20 20:26:20 mju Exp $
 *
 * $Log: uishape.h_v $
 * Revision 1.11  2011/10/20 20:26:20  mju
 * Remove editInTNTserver flag.
 *
 * Revision 1.10  2006/08/31 20:36:07  mju
 * Add applyChanges.
 *
 * Revision 1.9  2006/04/13 16:26:49  dwilliss
 * doxygen problems
 *
 * Revision 1.8  2005/10/17 15:30:08  mju
 * Use gre namespace.
 *
 * Revision 1.7  2005/10/14 17:33:34  mju
 * GRE namespace and treeview.
 *
 * Revision 1.3  2003/06/17 21:19:18  scowan
 * Updated ctrl panel dlg for non-modal operation.
 *
 * Revision 1.1  2003/06/06 22:07:47  scowan
 * Initial revision
 * \endif
 */

#ifndef	INC_GRE_UISHAPE_H
#define	INC_GRE_UISHAPE_H

#ifndef	INC_GRE_LRSHAPE_H
#include <gre/lrshape.h>
#endif

#ifdef USE_MGUI

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

namespace GRE {
//===================================================================================================================
//! Dialog for modifying vector layer display settings.
class LAYER_SHAPE::DISPPARM::DLG : public MGUI::DLGSHELL {
	public:

		//! Constructor.
		DLG (
			LAYER_SHAPE* ShapeLayer,
			bool Include3D
			);

		//! Destructor.
		virtual ~DLG (
			);

		//! Get 'mask' for which LAYERDRAWFLAG values are settable on dialog.
		LAYERDRAWFLAGS GetLayerDrawFlagMask (
			) const { return (LAYERDRAWFLAG_None); }

		//! Get current LAYERDRAWFLAGS settings.
		LAYERDRAWFLAGS GetLayerDrawFlags (
			) const;

		//! Get current DISPPARM values.
		const LAYER_SHAPE::DISPPARM& GetValues (
			) const;

		//! Set DISPPARM and LAYERDRAWFLAGS values.
		ERRVALUE SetValues (
			const LAYER_SHAPE::DISPPARM& values,
			LAYERDRAWFLAGS LayerDrawFlags
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		LAYER_SHAPE *m_layer;

		void ApplyChanges (bool ForceRedraw);

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnApply ();
		virtual void v_OnClose ();
		virtual void v_OnOK ();
		virtual void v_OnOpen ();

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	// USE_MGUI

#endif	// INC_GRE_UISHAPE_H

