/**
 * \file gre/viewlegacy.h
 * \brief GRE VIEW_LEGACY classes
 *
 * \if NODOC
 * $Id: viewlegacy.h_v 1.25 2011/11/03 14:25:22 mju Exp $
 *
 * $Log: viewlegacy.h_v $
 * Revision 1.25  2011/11/03 14:25:22  mju
 * Use strtodeg hdr.
 *
 * Revision 1.24  2011/10/31 17:35:14  mju
 * Add useStereo parm to v_getDrawingDeviceMemory.
 *
 * Revision 1.23  2011/10/20 13:34:02  mju
 * Remove redundant hdrs.
 *
 * Revision 1.22  2008/04/28 13:23:16  mju
 * Remove obsolete prototypes.
 *
 * Revision 1.21  2008/02/18 15:50:34  mju
 * Move legacy_savesettings defns here.
 *
 * Revision 1.20  2008/02/01 16:38:03  mju
 * Use private macroscriptcontext class.
 *
 * Revision 1.19  2008/01/25 15:35:54  mju
 * Make v_updateScriptUI virtual.
 *
 * Revision 1.18  2007/11/12 14:43:28  mju
 * Use stdafx hdr for mfc.
 *
 * Revision 1.17  2007/11/12 13:49:59  mju
 * Don't have statusbar member unless use_mgui.
 *
 * Revision 1.16  2007/11/09 18:38:57  mju
 * Add statusbar.
 *
 * Revision 1.15  2007/10/15 19:19:20  mju
 * Add setLatLonFormat variant using dmstype.
 *
 * Revision 1.14  2007/10/12 15:15:15  mju
 * Move coordsource enum here and rename.
 *
 * Revision 1.13  2007/10/11 17:47:19  mju
 * Add datatip overrides.
 *
 * Revision 1.12  2007/10/08 14:54:16  mju
 * Exclude v_getDlgParent unless use_mgui.
 *
 * Revision 1.11  2007/10/08 12:43:57  mju
 * No v_testCancel unless X.
 *
 * Revision 1.10  2007/10/05 21:18:47  mju
 * *** empty log message ***
 *
 * Revision 1.9  2007/10/05 19:35:25  mju
 * Exclude some inlines unless x_native.
 *
 * Revision 1.8  2007/10/05 17:58:12  mju
 * More more legacy stuff here.
 *
 * Revision 1.7  2007/10/04 17:56:01  mju
 * Move toollist to view_legacy.
 *
 * Revision 1.6  2007/10/04 17:54:10  mju
 * Move X-specific members to view_legacy.
 *
 * Revision 1.5  2007/10/02 14:18:46  mju
 * Move view_legacy_statusinfo to gre namespace so Unix not confused.
 *
 * Revision 1.4  2007/10/02 13:31:41  mju
 * More scope resolution.
 *
 * Revision 1.3  2007/10/02 12:40:01  mju
 * Try to fix for Unix.
 *
 * Revision 1.2  2007/10/01 20:44:45  mju
 * Move most legacy-specific methods here.
 *
 * Revision 1.1  2007/09/28 21:59:39  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_VIEWLEGACY_H
#define	INC_GRE_VIEWLEGACY_H

#ifndef INC_GRE_VIEW_H
	#include <gre/view.h>
#endif

#ifndef X_H
	#include <X11/X.h>
	#define X_H
#endif

#ifndef INC_MI32_GRTREE_H
	#include <mi32/grtree.h>
#endif

#ifndef INC_MI32_STRTODEG_H
	#include <mi32/strtodeg.h>
#endif

#if !defined(INC_MGUI_STATUSBAR_H) && defined(USE_MGUI)
	#include <mgui/statusbar.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
namespace MGD {
	class DEVICE_X;
	class DEVICE_X32;
	class DEVICE_MEM32;
	}
#endif

//===================================================================================================================
namespace GRE {

//! Base class for tools attached to view.
class TOOL {
	public:

		//! Standard constructor.
		TOOL (
			GRE::VIEW_LEGACY *view,								//!< View to associate tool with
	#ifdef WIN32_NATIVE
			const char *toolname							//!< Tool name string, cannot be NULL
	#else
			const char *toolname,						//!< Tool name string, cannot be NULL
			const char *iconname,						//!< Tool icon name
			const char *icongroup=NULL,				//!< Tool icon group
			const char *helpcode=NULL					//!< Help code for context-sensitive help
	#endif
			);

		//! Constructor with interface component IDs.
		TOOL (
			GRE::VIEW_LEGACY *view,								//!< View to associate tool with
			ICID icidmenu,									//!< Interface component ID for menu customization
			ICID icidicon,									//!< Interface component ID for ToolBar customization
	#ifdef WIN32_NATIVE
			const char *toolname							//!< Tool name string, cannot be NULL
	#else
			const char *toolname,						//!< Tool name string, cannot be NULL
			const char *iconname,						//!< Tool icon name
			const char *icongroup=NULL,				//!< Tool icon group
			const char *helpcode=NULL					//!< Help code for context-sensitive help
	#endif
			);

		virtual ~TOOL ();

		//! Activate this tool.
		void Activate (
			);

		//! Deactivate this tool.
		void Deactivate (
			);

	#ifdef X_NATIVE
		//! Get "help code" associated with tool.
		const char* GetHelpCode (
			) const { return (m_HelpCode); }

		//! Get "icon group" for tool.
		const char* GetIconGroup (
			) const { return (m_IconGroup); }

		//! Get "icon name" for tool.
		const char* GetIconName (
			) const { return (m_IconName); }
	#endif

		//! Get tool name.
		const char* GetName (
			) const { return (m_ToolName); }

		//! Get view this tool is attached to.
		GRE::VIEW_LEGACY* GetView (
			) const { return (m_view); }

	#ifdef X_NATIVE
		//! Get widget for item on view's "Tool" menu.
		Widget GetMenuWidget (
			) const { return (m_MenuWidget); }

		//! Get widget for button on view's toolbar.
		Widget GetToolbarWidget (
			) const { return (m_ToolbarWidget); }
	#endif

		//! Determine if tool is currently disabled.
		bool IsDisabled (
			) const { return (m_IsDisabled); }

		//! Determine if tool is implemented via script.
		//! This should only be overridden by the script tool.
		virtual bool IsScript (
			) const;

	#ifdef X_NATIVE
		//! Setup BUTTONITEM for use in toolbar.
		void SetupButtonItem (
			BUTTONITEM& item
			);

		//! Setup MENUITEM to use on menu to activate tool.
		void SetupMenuItem (
			MENUITEM& item
			);

		//! Set sensitivity of tool menu item and button.
		void SetSensitive (
			bool sensitive = true
			);
	#endif

		//! Suspend tool (during screen update).
		//! @return true if suspended, false if was already suspended
		bool Suspend (
			);

		//! Resume tool (after screen update)
		void Resume (
			);

	protected:

		enum TOOLSTATE {
			STATE_NotInitialized = 0,
			STATE_Inactive,
			STATE_Active,
			STATE_NoLayers,
			STATE_Suspended
			};

		GRE::VIEW_LEGACY* m_view;								//!< View tool is contained in

		//! Get current tool state.
		TOOLSTATE GetState (
			) const { return (m_state); }

		void SetName (
			const MISTRING& name
			) { m_ToolNameStr = name; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		char *m_ToolName;
	#ifdef X_NATIVE
		char *m_IconName;
		char *m_IconGroup;
		char *m_HelpCode;
	#endif
		TOOLSTATE m_state;
		bool m_IsDisabled;
		ICID m_icidmenu;								//!< ID for tool menu item
		ICID m_icidicon;								//!< ID for tool icon on ToolBar
	#ifdef X_NATIVE
		Widget m_MenuWidget;
		Widget m_ToolbarWidget;
	#endif
		MISTRING m_ToolNameStr;

		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables

		virtual bool DoActivate () = 0;
		virtual void DoDeactivate () = 0;
		virtual bool DoInitialize ();
		virtual void DoSuspend ();
		virtual void DoResume ();
	};

}	// End namespace GRE

#ifndef GENERATING_DOXYGEN_OUTPUT
//! Implement old-style tools.
class _MDISPTOOL : public GRE::TOOL {
	public:

		void *privptr;
		GRE::VIEW_LEGACY *view;

		_MDISPTOOL (
			GRE::VIEW_LEGACY *view,
			const char *toolname,
			const char *iconname,
			const char *icongroup,
			const char *helpcode,
			int (*Init)(_MDISPTOOL*),
			int (*Stop)(_MDISPTOOL*),
			int (*Activate)(_MDISPTOOL*),
			int (*Deactivate)(_MDISPTOOL*),
			int (*Suspend)(_MDISPTOOL*),
			int (*Resume)(_MDISPTOOL*)
			);

		_MDISPTOOL (
			GRE::VIEW_LEGACY *view,
			const char *toolname,
			const char *iconname,
			const char *icongroup,
			const char *helpcode,
			ICID icidmenu,
			ICID icidicon,
			int (*Init)(_MDISPTOOL*),
			int (*Stop)(_MDISPTOOL*),
			int (*Activate)(_MDISPTOOL*),
			int (*Deactivate)(_MDISPTOOL*),
			int (*Suspend)(_MDISPTOOL*),
			int (*Resume)(_MDISPTOOL*)
			);

		virtual ~_MDISPTOOL ();

	private:

		virtual bool DoActivate ();
		virtual void DoDeactivate ();
		virtual bool DoInitialize ();
		virtual void DoSuspend ();
		virtual void DoResume ();

		int (*OldInit)(_MDISPTOOL*);
		int (*OldStop)(_MDISPTOOL*);
		int (*OldActivate)(_MDISPTOOL*);
		int (*OldDeactivate)(_MDISPTOOL*);
		int (*OldSuspend)(_MDISPTOOL*);
		int (*OldResume)(_MDISPTOOL*);
	};
typedef _MDISPTOOL *MDISPTOOL;
#endif // GENERATING_DOXYGEN_OUTPUT

namespace GRE {

//===================================================================================================================

enum VIEW_LEGACY_COORDSOURCE {
	VIEW_LEGACY_COORDSOURCE_None,
	VIEW_LEGACY_COORDSOURCE_Mouse,
	VIEW_LEGACY_COORDSOURCE_ViewCenter,
	VIEW_LEGACY_COORDSOURCE_Entered,
	VIEW_LEGACY_COORDSOURCE_GPS,
	VIEW_LEGACY_COORDSOURCE_COUNT
	};

//===================================================================================================================

class VIEW_LEGACY_SAVESETTINGS : public SERIALIZABLE {
	public:

		VIEW_LEGACY_SAVESETTINGS ();
		virtual ~VIEW_LEGACY_SAVESETTINGS ();

		const SPATREF::COORDREFSYS& GetStatusPos1_CRS (
			) const { return (m_StatusPos1.m_CoordRefSys); }

		DMSTYPE GetStatusPos1_LatLonFormat (
			) const { return (m_StatusPos1.m_LatLonFormat); }

		const SPATREF::COORDREFSYS& GetStatusPos2_CRS (
			) const { return (m_StatusPos2.m_CoordRefSys); }

		DMSTYPE GetStatusPos2_LatLonFormat (
			) const { return (m_StatusPos2.m_LatLonFormat); }

		bool GetShowStatusPos1 (
			) const { return (m_StatusPos1.m_Show); }

		bool GetShowStatusPos2 (
			) const { return (m_StatusPos2.m_Show); }

		void IniRead (INIHANDLE handle, const char *inigroup);
		void IniWrite (INIHANDLE handle, const char *inigroup);

		void SetShowStatusPos1 (
			bool show
			) { m_StatusPos1.m_Show = show; }

		void SetShowStatusPos2 (
			bool show
			) { m_StatusPos2.m_Show = show; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		struct STATUSPOS {
			bool m_Show;
			UINT8 m_UnitCode;
			SPATREF::COORDREFSYS m_CoordRefSys;
			DMSTYPE m_LatLonFormat;
			static const SERIALIZERITEM* SerialGetItemDef ();
			STATUSPOS ();
			};

		STATUSPOS m_StatusPos1;
		STATUSPOS m_StatusPos2;

		static const SERIALIZERITEM* SerialGetItemDef ();

		//! Overrides from SERIALIZABLE.
		virtual ERRVALUE SerialRead (SERIALIZER& serializer);
		virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;

	
		friend class VIEW;
		friend struct STATUSPOS;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
#ifndef GENERATING_DOXYGEN_OUTPUT

class VIEW_LEGACY_STATUSINFO {
	public:
		class POS {
			public:
				POS ();
				~POS ();
				void GetCoordString (MISTRING& string, const DPOINT2D& pt, const SPATREF::COORDREFSYS& PointCRS, bool UseLatLonIfNoProjSet = true) const;
				DEGTOSTRFLAGS GetLatLonFormat () const { return (m_LatLonFormat); }
				const SPATREF::COORDREFSYS& GetCoordRefSys () const { return (m_CoordRefSys); }
				void SetLatLonFormat (DEGTOSTRFLAGS LatLonFormat);
				void SetLatLonFormat (DMSTYPE LatLonFormat);
				void SetCoordRefSys (const SPATREF::COORDREFSYS& CoordRefSys) { m_CoordRefSys = CoordRefSys; }
			private:
				SPATREF::COORDREFSYS m_CoordRefSys;
				DEGTOSTRFLAGS m_LatLonFormat;		// Latitude-longitude format
			};

		VIEW_LEGACY_STATUSINFO ();
		void ClearPos (VIEW_LEGACY_COORDSOURCE CoordSource = VIEW_LEGACY_COORDSOURCE_None);
	#ifdef X_NATIVE
		void Create (Widget pwidget, GRE::VIEW *view, Arg *arglist, int argcount, const GRE::VIEW_LEGACY_SAVESETTINGS& settings);
	#endif
		void DisplayPos (VIEW_LEGACY_COORDSOURCE CoordSource, GRE::GROUP *CoordGroup, const DPOINT2D& pt);
		void DisplayPos (VIEW_LEGACY_COORDSOURCE CoordSource, const DPOINT2D& pt, const SPATREF::COORDREFSYS& PointCRS, bool UseLatLonIfNoProjSet = true);
	#ifdef X_NATIVE
		double GetEditScale () const { return (m_EditScale); }
		double GetEditZoom () const { return (m_EditZoom); }
	#endif
		DEGTOSTRFLAGS GetLatLonFormat (int idx) const
			{ return (GetPos(idx).GetLatLonFormat()); }
		const POS& GetPos (int idx) const
			{ return ((idx != 1) ? m_Pos1 : m_Pos2); }
		POS& GetPos (int idx)
			{ return ((idx != 1) ? m_Pos1 : m_Pos2); }
//		UINT8 GetUnitCode (int idx) const
//			{ return (GetPos(idx).GetUnitCode()); }
	#ifdef X_NATIVE
		Widget GetWidget () const { return (m_container); }
	#endif
		bool HasPosVisible () const;
		bool IsPosVisible (int idx) const;
		void SetCoordSource (VIEW_LEGACY_COORDSOURCE CoordSource);
		void SetPosVisible (int idx, bool visible);
		void UpdateZoomScaleDisplay (GRE::VIEW_LEGACY *view);

	private:
	#ifdef X_NATIVE
		Widget m_container;
		Widget m_zoomP;							// Prompt for zoom factor
		Widget m_mapscaleP;						// Prompt for map scale
		Widget m_PanToPointPB;					// PushButton for panning to user-specified position
		Widget m_CoordSourceIcon;				// Icon label for coordinate type indication
		Widget m_Coord1XeT;
		Widget m_Coord2XeT;
		MGUI::CTRL_TOOLTIP *m_pCoord1ToolTip;
		MGUI::CTRL_TOOLTIP *m_pCoord2ToolTip;
		double m_EditZoom;
		double m_EditScale;
	#endif
		VIEW_LEGACY_COORDSOURCE m_CoordSource;				// Current coordinate source for icon label
		POS m_Pos1;
		POS m_Pos2;
	};
#endif // GENERATING_DOXYGEN_OUTPUT

//===================================================================================================================

#define	LEGENDVIEW_SHOW_NotSet	0
#define	LEGENDVIEW_SHOW_None		1
#define	LEGENDVIEW_SHOW_Left		2
#define	LEGENDVIEW_SHOW_Right	3

#define	GRE_MaxPrevViews	10					//!< Maximum number of "previous views" to retain

//===================================================================================================================

class VIEW_LEGACY : public VIEW {
	public:

		//! Flags for view creation.
		enum CREATEFLAGS {
			CREATEFLAG_None =							0x00000000,
			// Flags must match base class.
			CREATEFLAG_3D =							GRE::VIEW::CREATEFLAG_3D,						//!< View is 3D
			CREATEFLAG_NoCallCreateCB =			GRE::VIEW::CREATEFLAG_NoCallCreateCB,		//!< Don't call callback list when object created
			CREATEFLAG_NoRenderSeparate =			GRE::VIEW::CREATEFLAG_NoRenderSeparate,	//!< Don't render to separate memory buffers
			// Flags specific to this class.
			CREATEFLAG_AllowGeoLock =				0x00000800,	//!< Allow GeoLocking
			CREATEFLAG_DestroyOnClose =			0x00001000,	//!< Destroy view when View/Close selected
			CREATEFLAG_DialogModal =				0x00002000,	//!< View is contained in modal dialog
			CREATEFLAG_Locator =						0x00004000,	//!< Create locator view
			CREATEFLAG_NoCancelButton =			0x00008000,	//!< Don't put "Cancel" button at bottom of window
			CREATEFLAG_NoCloseOption =				0x00010000,	//!< Don't include "Close" option on view menu
			CREATEFLAG_NoDftAddIcons =				0x00020000,	//!< Don't put default "add" icon(s) on toolbar
			CREATEFLAG_NoDftDrawIcons =			0x00040000,	//!< Don't create default "draw" icons when creating window
			CREATEFLAG_NoDftMarkIcons =			0x00080000,	//!< Don't create default "mark" icons when creating window
			CREATEFLAG_NoDftViewIcons =			0x00100000,	//!< Don't create default "view" icons when creating window
			CREATEFLAG_NoIconBar =					0x00200000,	//!< Don't create iconbar
			CREATEFLAG_NoLegendView =				0x00400000,	//!< Don't create LegendView
			CREATEFLAG_NoRedrawButton =			0x00800000,	//!< Don't put "Redraw" button at bottom of window
			CREATEFLAG_NoScalePosLine =			0x01000000,	//!< Don't create zoom/scale/position line at bottom of view window
			CREATEFLAG_NoScrollbars =				0x02000000,	//!< Don't create scroll bars
			CREATEFLAG_NoSelectTool =				0x04000000,	//!< Don't add "Select" tool
			CREATEFLAG_NoSkipButton =				0x08000000,	//!< Don't put "Skip" button at bottom of window
			CREATEFLAG_NoStatusLine =				0x10000000,	//!< Don't create "status" line
			CREATEFLAG_NoStdTools =					0x20000000,	//!< Don't add any standard tools
			CREATEFLAG_NoToolBoxTool =				0x40000000,	//!< Don't add "ToolBox" tool
			CREATEFLAG_Temporary =					0x80000000,	//!< Object is temporary
			};
		enum ICONVIEW {
			ICONVIEW_Full =				0x00000001,
			ICONVIEW_Previous =			0x00000002,
			ICONVIEW_PreviousZoom =		0x00000004,
			ICONVIEW_ZoomIn =				0x00000008,
			ICONVIEW_ZoomOut =			0x00000010,
			ICONVIEW_ZoomToGroup =		0x00000020,
			ICONVIEW_ZoomToLayer =		0x00000040,
			ICONVIEW_ZoomToMaxDetail =	0x00000080,
			ICONVIEW_ZoomToLocation =	0x00000100,
			ICONVIEW_Zoom1X =				0x00000200,
			ICONVIEW_Zoom2X =				0x00000400,
			ICONVIEW_Zoom3X =				0x00000800,
			ICONVIEW_Locator =			0x00001000,
			ICONVIEW_Snapshot =			0x00002000,
			ICONVIEW_GeoLock =			0x00004000,
			ICONVIEW_GoogleEarth =		0x00008000,
			ICONVIEW_Stereo =				0x00010000,
			ICONVIEW_Default =			ICONVIEW_Full | ICONVIEW_Previous | ICONVIEW_PreviousZoom | ICONVIEW_ZoomIn | ICONVIEW_ZoomOut | ICONVIEW_ZoomToLayer | ICONVIEW_ZoomToGroup | ICONVIEW_ZoomToLocation | ICONVIEW_ZoomToMaxDetail | ICONVIEW_Zoom1X | ICONVIEW_Locator | ICONVIEW_Snapshot | ICONVIEW_GeoLock | ICONVIEW_GoogleEarth | ICONVIEW_Stereo
			};

		//! Find next legacy view containing specified group
		static VIEW_LEGACY* FindNextLegacyOfGroup (
			const GRE::GROUP *group,			//!< Group to find view of
			VIEW *view = 0							//!< View to start search after, 0 to check first view
			);

		//! Find next legacy view containing specified layer.
		static VIEW_LEGACY* FindNextLegacyOfLayer (
			const GRE::LAYER *layer,			//!< Layer to find view of
			VIEW *view = 0							//!< View to start search after, 0 to check first view
			);

		// Install SML class, called by internal SML function.
		static void SML_ClassInstall (
			SML::CONTEXT *context
			);

	#if defined(X_NATIVE) && !defined(GENERATING_DOXYGEN_OUTPUT)
		// Internal helper method for creating view from SML.
		static VIEW* SML_CreateX (
			int numargs,
			SML::ARG *args,
			SML::CONTEXT *context,
			CREATEFLAGS createflags = CREATEFLAG_None
			);
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
		// Internal helper method for getting creation flags.
		static SML::FLAG* SML_GetCreateFlags (
			);
	#endif

		//! Constructor.
		VIEW_LEGACY ();

		//! Destructor.
		virtual ~VIEW_LEGACY ();

	#ifdef X_NATIVE
		//! Add default "drawing" icons (redraw, skip, stop).
		void AddDrawIcons (
			);

		//! Add icon buttons to icon row.
		Widget AddIconRowItems (
			const BUTTONITEM *items,					//!< Items to add
			const char *icongroup = 0,					//!< Icon group
			void *cbdata = 0,								//!< Button item callback data if NULL in item
			void *userdata = 0							//!< Button item userdata if NULL in item
			);

		//! Add icon pushbuttons to toolbar for MacroScripts.
		void AddMacroScriptIcons (
			);

		//! Add menu items for MacroScripts
		void AddMacroScriptMenuItems (
			);

		//! Add additional items to "Options" menu (must be added before menubar created)
		ERRVALUE AddMenuItemsOptions (
			const MENUITEM *newitems					//!< Array of items to add, terminated by null item
			);

		//! Add icon togglebuttons to toolbar for all defined tools.
		void AddToolIcons (
			bool addscripts = true						//!< Add ToolScripts and MacroScripts to toolbar
			);

		//! Add "view" icons (previous, zoom in, zoom out, zoom 1X, etc).
		void AddViewIcons (
			ICONVIEW icons = ICONVIEW_Default		//! Which icons to add.
			);
	#endif

		//! Clear previous view list.
		void ClearPreviousViewList (
			) { m_PrevViewIndex = 0; return; }

	#ifdef X_NATIVE
		//! Create menubar with specified set of items.
		//! @return RowColumn widget for menubar.
		Widget CreateMenuBar (
			Widget form,									//!< Form to contain menubar
			MENUBARITEM *menubar							//!< Menu bar items
			);

		//! Create menubar with default set of items.
		//! @return RowColumn widget for menubar.
		Widget CreateMenuBar (
			Widget form										//!< Form to contain menubar
			);

		//! Create "form" in X.
		ERRVALUE CreateX (
			VIEWABLE *viewable,
			Widget pwidget,
			Arg *arglist,
			int argcount,
			int iwidth,
			int iheight,
			const char *sizefield = 0,				//!< Field from .ini for retrieving window size
			CREATEFLAGS createflags = CREATEFLAG_None,
			const VIEW_LEGACY_SAVESETTINGS *pSettings = 0
			);
	#endif

		//! Set DataTip popup delay.
		void DataTipSetDelay (
			INT32 delay										//!< DataTip popup delay in milliseconds
			);

		//! Set distance cursor must move to hide DataTip.
		void DataTipSetPixelDelta (
			int PixelDelta									//!< Movement distance in screen pixels
			);

		//! Edit interface customization settings for views of this type.
		void EditCustomization (
			);

		//! Return current active tool for view
		TOOL* GetActiveTool (
			) const { return (m_ActiveTool); }

	#ifdef X_NATIVE
		//! Get DrawingArea widget for view
		Widget GetDrawingArea (
			) const { return (m_da); }

		//! Get "form" containing DrawingArea and LegendView.
		Widget GetDrawingAreaForm (
			) const { return (m_lvswform); }

		//! Get MGD::DEVICE for 'view-in-view'.
		MGD::DEVICE_X32* GetDrawingDeviceViewInView (
			) const { return (m_pDeviceViewInView); }

		//! Get X Pixmap.
		//! @return 0 if none exists
		Pixmap GetDrawingPixmap (
			) const { return (m_pixmap); }

		//! Get main "form" widget for this view.
		Widget GetForm (
			) const { return (m_form); }

		//! Get BUTTONITEM array for "Add" icons on toolbar.
		//! @return Cast of array to BUTTONITEM* for convenience.
		const BUTTONITEM* GetIconItemAdd (
			BUTTONITEMARRAY& items			//!< Array to contain items
			) const;

		//! Get BUTTONITEM array for "Draw" icons on toolbar.
		//! @return Cast of array to BUTTONITEM* for convenience.
		const BUTTONITEM* GetIconItemDraw (
			BUTTONITEMARRAY& items			//!< Array to contain items
			) const;

		//! Get BUTTONITEM array for "Tool" icons on toolbar.
		//! This does not include items for custom script tools.
		//! @return Cast of array to BUTTONITEM* for convenience.
		const BUTTONITEM* GetIconItemTool (
			BUTTONITEMARRAY& items			//!< Array to contain items
			) const;

		//! Get BUTTONITEM array for "View" icons on toolbar.
		//! @return Cast of array to BUTTONITEM* for convenience.
		const BUTTONITEM* GetIconItemView (
			BUTTONITEMARRAY& items,			//!< Array to contain items
			ICONVIEW icons = ICONVIEW_Default		//!< Which icons to include
			) const;

		//! Get "view" toolbar icon.
		//! @return Icon widget or NULL if does not exist.
		Widget GetIconView (
			ICONVIEW icon									//!< Icon to get widget for
			) const;

		//! Get GPS menu items.
		//! @return Internal array
		const MENUITEM* GetMenuItemGPS (
			);

		//! Get "HotKeys" menu items.
		//! @return Internal array
		const MENUITEM* GetMenuItemHotKeys (
			);

		//! Get "LegendView" menu items.
		//! @return Internal array
		const MENUITEM* GetMenuItemLegendView (
			);

		//! Get "Options" menu items.
		//! @return Cast of array to MENUITEM* for convenience.
		const MENUITEM* GetMenuItemOptions (
			MENUITEMARRAY& items				//!< Array of items returned
			);

		//! Get MENUITEM array for "Tool" menu.
		//! @return Cast of array to MENUITEM* for convenience.
		const MENUITEM* GetMenuItemTool (
			MENUITEMARRAY& items				//!< Array of items returned
			) const;

		//! Get "View" menu items.
		//! @return Cast of array to MENUITEM* for convenience.
		const MENUITEM* GetMenuItemView (
			MENUITEMARRAY& items				//!< Array of items returned
			);

		//! Get Widget for item on view's menubar.
		//! @return Widget or NULL if no matching item.
		Widget GetMenuWidget (
			const char* menupath							//!< Path to menu item (eg. "/view/redraw")
			) const;
	#endif

		//! Get next legacy view of same object as this view.
		//! @return next view or 0 if no more views.
		VIEW_LEGACY* GetNextLegacy (
			) const;

		VIEW_LEGACY* GetNextLegacyOfAll (
			) const;

	#ifdef X_NATIVE
		//! Get widget for view's ToolBar.
		Widget GetToolBar (
			) const { return (m_ToolBarRC); }
	#endif

	#ifdef X_NATIVE

		//! Enable or disable redraw of LegendView.
		void LegendEnableRedraw (
			bool enable
			);

		UINT8 LegendGetShowMode (
			) const { return (m_LegendShowMode); }

		void LegendHideAllLegends (
			);

		void LegendRedraw (
			bool updatesizes = false
			);

		void LegendSetShowLayerTypeIcon (
			bool ShowLayerType
			) { m_LegendShowLayerType = ShowLayerType; }

		void LegendSetShowMode (
			UINT8 ShowMode
			);

		void LegendSetVisiblityIndicator (
			UINT8 VisInd
			) {
			if (VisInd != m_LegendVisInd) {
				m_LegendVisInd = VisInd;
				LegendRedraw();
				}
			return;
			}

		void LegendShowAllLegends (
			);
	#endif

		//! Clear coordinate display in scale/position report.
		void ScalePosClearCoord (
			VIEW_LEGACY_COORDSOURCE CoordSource = VIEW_LEGACY_COORDSOURCE_None
			) { m_StatusInfo.ClearPos(CoordSource); }

		void ScalePosDisplayCoord (
			VIEW_LEGACY_COORDSOURCE CoordSource,
			const DPOINT2D& point,
			const SPATREF::COORDREFSYS& PointCRS
			) { m_StatusInfo.DisplayPos(CoordSource,point,PointCRS); }

		void ScalePosGetCoordString (
			int idx,
			const DPOINT2D& point,						//!< Point in 'view' coordinates
			MISTRING& string								//!< String returned
			) const;

		//! Get latitude-longtitude position report format.
		DEGTOSTRFLAGS ScalePosGetLatLonFormat (
			int idx
			) const { return (m_StatusInfo.GetLatLonFormat(idx)); }

		//! Get coordinate reference system for scale/position report.
		//! If no fixed projection is set the CRS of the first suitable group will be returned.
		const SPATREF::COORDREFSYS& ScalePosGetCoordRefSys (
			int idx
			) const;

	#ifdef X_NATIVE
		//! Determine if statusbar position indicator is visible.
		bool ScalePosIsPosVisible (
			int idx
			) const { return (m_StatusInfo.IsPosVisible(idx)); }
	#endif

		void ScalePosSetCoordSource (
			VIEW_LEGACY_COORDSOURCE CoordSource
			) { m_StatusInfo.SetCoordSource(CoordSource); }

		void ScalePosSetLatLonFormat (
			int idx,
			DEGTOSTRFLAGS LatLonFormat
			) { GetStatusInfoPos(idx).SetLatLonFormat(LatLonFormat); };

		//! Set whether statusbar position indicator is visible.
		void ScalePosSetPosVisible (
			int idx,
			bool visible
			) { m_StatusInfo.SetPosVisible(idx,visible); }

		//! Set projection of coordinates displayed on scale/position line.
		void ScalePosSetCoordRefSys (
			int idx,
			const SPATREF::COORDREFSYS& CoordRefSys
			) { GetStatusInfoPos(idx).SetCoordRefSys(CoordRefSys); }

	#ifdef X_NATIVE
		//! Set scale/position line visibility
		void ScalePosSetVisible (
			bool visible									//!< true to show, false to hide
			);
	#endif

	#ifdef X_NATIVE
		void SetDrawingArea (
			Widget da
			) { m_da = da; }
	#endif

		void SetEditCustomFunc (
			void (*EditCustomFunc)(VIEW_LEGACY*)
			) { m_EditCustomFunc = EditCustomFunc; }

		//! Set input (mouse/keyboard) focus to view.
		//! Will activate window, and direct keyboard input to drawing canvas control.
		void SetInputFocusTo (
			);

		//! Set visibility of "message" line at bottom of view.
		void SetMessageVisible (
			bool visible									//!< 'true' to show, 'false' to hide
			);

		void SetPreviousView (
			);

		void SetPreviousZoom (
			);

		//! Attach tool to view.
		//! This is done by the tool constructor and does not need to be called directly.
		void ToolAttach (
			TOOL *tool									//!< Tool to attach
			);

		//! Find tool with given name if any.
		TOOL* ToolFindByName (
			const char* toolname							//!< Tool name
			) const;

		//! Get the active tool.
		//! @return Pointer to active tool or NULL if no tool active.
		TOOL* ToolGetActive (
			) const { return (m_ActiveTool); }

		//! Determine if specified tool is active (by tool).
		bool ToolIsActive (
			TOOL* tool									//!< Tool
			) const { return (m_ActiveTool == tool && tool != NULL); }

		//! Determine if specified tool is active (by name).
		bool ToolIsActive (
			const char *toolname							//!< Tool name
			) const { if (m_ActiveTool) return (!strcmp(m_ActiveTool->GetName(),toolname)); else return (false); }

		//! Resume tool after suspended.
		//! @see ToolSuspend.
		void ToolResume (
			) { if (m_ActiveTool) m_ActiveTool->Resume(); }

		//! Set active tool for view (by tool).
		void ToolSet (
			TOOL* tool									//!< Tool
			) { if (!tool) tool = (*m_ToolList.Begin()); if (tool) tool->Activate(); }

		//! Set active tool for view (by name).
		int ToolSet (
			const char* toolname							//!< Tool name
			);

		//! Set tool to default.
		void ToolSetDefault (
			);

		//! Suspend tool (during screen update).
		//! Before drawing on a view the current graphic tool must be suspended.
		//! Normally this is done automatically by the GRE system, but in cases
		//! where direct updates are required this method may be called directly.
		//! The caller should use the Resume() method if and only if the Suspend()
		//! method returns TRUE upon completion of the drawing operation.
		//! @return true if suspended, false if was already suspended.
		//! @see ToolResume.
		int ToolSuspend (
			) { if (m_ActiveTool) return (m_ActiveTool->Suspend()); else return (0); }

	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		// SML interface methods.
		static const SML::CLASSMEMBER* SML_GetClassMembers();
		#endif

	private:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		friend class GRE::VIEW_LEGACY_STATUSINFO;
		friend class GRE::TOOL;

	#ifdef X_NATIVE
		class MACROSCRIPTCONTEXT;

		GRAPHICTREE *m_Legendgtree;
		Widget m_LegendSash;
		Widget m_LegendLayerPopupMenu;			// Popup menu for layer
		Widget *m_LegendLayerPopupMenuWidgets;	// Actual widgets on popup menu for manage/unmanage
		Widget m_LegendGroupPopupMenu;			// Popup menu for group
		Widget *m_LegendGroupPopupMenuWidgets;	// Actual widgets on popup menu for manage/unmanage
		GRE::LAYER *m_LegendCursorLayer;			// Layer cursor is in for popup menu use
		GRE::GROUP *m_LegendCursorGroup;			// Group cursor is in for popup menu use
		UINT8 m_LegendShowMode;
		UINT8 m_LegendVisInd;						// Visibility indicator LEGENDVIEW_VISIND_...
		bool m_LegendShowLayerType;				// Show icon for layer type
		bool m_LegendDftLegendHidden;				// Default to have legend hidden
		Display* m_xdisplay;							// X Display pointer
		Widget m_form;									// Outer form containing toolbar, ScrolledWindow, message lines
		Widget m_menubar;								// MenuBar widget (set by main control panel when view created)
		Widget m_lvswform;							// Inner form containing LegendView and ScrolledWindow
		Widget m_sw;									// ScrolledWindow containing DrawingArea
		Widget m_hsb;									// Horizontal ScrollBar widget
		Widget m_vsb;									// Vertical ScrollBar widget
		Widget m_da;									// DrawingArea widget
		Widget m_ToolBarRC;							// RowColumn widget for toolbar
		mutable Widget m_RedrawPB;					// Redraw PushButton
		mutable Widget m_RedrawOnAnyChangeTB;	// ToggleButton for redraw on any change
		mutable Widget m_SkipPB;					// PushButton for skipping to next layer during redraw
		mutable Widget m_CancelPB;					// PushButton for canceling redraw
		mutable Widget m_AddGroupMB;				// Menu-button for add-group icon
		mutable Widget m_AddLayerMB;				// Menu-button for add-layer icon
		Widget m_MenuMacrosButton;					// Menu PushButton for 'Macros'
		Widget m_MenuMacrosPane;					// MenuPane for 'Macros'
		Widget m_IconRowView;						// IconRow for "View" icons
		Widget m_IconRowTool;						// IconRow for "Tool" icons
		Widget m_IconRowMacroScript;				// IconRow for 'MacroScript' button
		Widget m_InfoForm;							// Form at bottom contain scale/pos and status info
		Widget m_MsgProgForm;						// Message/progress form
		MENUITEMARRAY *m_pMenuItemOptionsExtra;	// Extra 'Options' menu items
		MILIST<MACROSCRIPTCONTEXT*> m_MacroScriptContexts;
		Pixmap m_pixmap;								// Pixmap for entire view window
		MGD::DEVICE_X *m_sdisp;						// Generic display handle for drawing directly on screen
		MGD::DEVICE_X32 *m_mdisp;					// Generic display handle for drawing to memory/pixmap
		MGD::DEVICE_X32 *m_pDeviceViewInView;	// Device for view-in-view rendering
		XtIntervalId m_ResizeTimer;				// Timer to avoid resize flashing
	#endif

		MILIST<TOOL*> m_ToolList;					// List of tools attached to this view
		TOOL *m_ActiveTool;							// Tool currently in use
		UINT8 m_PrevViewIndex;
		struct PREVPOS {
			DPOINT2D center;							// Previous center point for View/Previous
			DPOINT2D zoom;								// Previous zoom factor for View/Previous
			} m_PrevViewPos[GRE_MaxPrevViews];
		DPOINT2D m_LastZoom;							// Previous zoom factor that's actually different
		void (*m_EditCustomFunc)(VIEW_LEGACY*);			// Function for editing interface customization
		GRE::VIEW_LEGACY_STATUSINFO m_StatusInfo;
	#ifdef USE_MGUI
		MGUI::CTRL_STATUSBAR m_StatusBar;
	#endif
		MGUI::CTRL_TOOLTIP *m_pDataTip;

		static void DftEditCustomFunc (
			VIEW_LEGACY *view
			);

	#ifdef X_NATIVE
		static void GRECB_ViewObject (
			const GRE::MSG *msg,
			VIEW_LEGACY *view
			);
	#endif


	#ifdef X_NATIVE
		void AddDftAddIcons ();
		void AddToolScript (const GRE::TOOLSCRIPT& script, Widget ToolMenuPane);
		void AddToolScripts ();
		void AutoResizeDialog (MDLGPARENT);
		ERRVALUE CreateDeviceMemPixmap ();
		void CreateMacroScriptContexts ();
		void DeleteMacroScriptContexts ();
		void EditMacroScripts ();
		void EditToolScripts ();
		MENUBARITEM* GetMenuBarItems () const;
	#endif

		const GRE::VIEW_LEGACY_STATUSINFO::POS& GetStatusInfoPos (int idx) const
			{ return (m_StatusInfo.GetPos(idx)); }

		GRE::VIEW_LEGACY_STATUSINFO::POS& GetStatusInfoPos (int idx)
			{ return (m_StatusInfo.GetPos(idx)); }

	#ifdef X_NATIVE
		void LegendAddGroupItem (
			GRE::GROUP *group,
			int pos = 0
			);

		void LegendAddLayerItem (
			GRE::LAYER *layer,
			int pos = 0
			);

		void LegendAddLayerItems (
			GRE::GROUP *group
			);

		void LegendCreate (
			);

		void LegendSetGroupLegendsVisible (
			GRE::GROUP *group,
			bool visible
			);

		void LegendSetupGroupLegend (
			GRE::GROUP *group
			);

		void LegendUpdateGroupLegend (
			GRE::GROUP *group,
			bool redraw = false
			);
	#endif

	#ifdef X_NATIVE
		int GetToolCount (
			) const { return (m_ToolList.GetNumItems()); }
	#endif

	#ifdef X_NATIVE
		void UpdateMacroScriptUI ();
		void UpdateSize ();
		void UpdateToolScriptUI ();
	#endif

	#ifdef X_NATIVE
		// CALLBACKS & MESSAGE HANDLERS
		static int LegendPopup_GroupStatus (VIEW_LEGACY*, void*, UINT16);
		static int LegendPopup_LayerStatus (VIEW_LEGACY*, void*, UINT16);
		static int StatusTestCancel (MISTATUSMANAGER*, void*);
		static void CB_ActionView (Widget, void*, void*);
		static void CB_LegendPopupMenuGroup (Widget, void*, void*);
		static void CB_LegendPopupMenuLayer (Widget, void*, void*);
		static void CB_MenuOptions (Widget, void*, void*);
		static void CB_ViewResize (Widget, void*, void*);
		static void EH_DrawArea (Widget, VIEW_LEGACY*, XEvent*, Boolean*);
		static void EH_ScrollH (Widget, VIEW_LEGACY*, XEvent*, Boolean*);
		static void EH_ScrollV (Widget, VIEW_LEGACY*, XEvent*, Boolean*);
		static void LegendMH_Group (GRAPHICTREEITEM_MSG *msg);
		static void LegendMH_GroupLegend (GRAPHICTREEITEM_MSG *msg);
		static void LegendMH_Layer (GRAPHICTREEITEM_MSG *msg);
		static void LegendPopup_GroupAction (VIEW_LEGACY*, void*, UINT16);
		static void LegendPopup_LayerAction (VIEW_LEGACY*, void*, UINT16);
		static void PCB_ChangeMapScale (Widget, void *vview);
		static void PCB_ChangeZoom (Widget, void *vview);
		static void TCB_RestoreBackground (Widget, DRECT2D*, void*);
		static void TP_ViewResize (void*, XtIntervalId*);
	#endif

		// VIEW overrides.
		virtual void v_DataTipSetDelay (INT32 delay);
		virtual void v_DataTipSetPixelDelta (int PixelDelta);
		virtual ERRVALUE v_Draw ();
		virtual ERRVALUE v_DrawThisOnly (GRE::DRAWFLAGS drawflags);
	#ifdef USE_MGUI
		virtual MDLGPARENT v_GetDlgParent () const;
	#endif
		virtual MGD::DEVICE* v_GetDrawingDeviceMemory (bool UseStereo) const;
		virtual MGD::DEVICE* v_GetDrawingDeviceScreen () const;
		virtual int v_GetViewNum () const;
		virtual void v_RestoreFromMemory (const LRECT2D *rect, bool UseScreenBuffer);
	#ifdef X_NATIVE
		virtual ERRVALUE v_TestCancel ();
	#endif
		virtual void v_UpdateMessage (const MISTRING& string);
	#ifdef X_NATIVE
		virtual void v_UpdateScriptUI ();
	#endif
		virtual void v_UpdateUI ();

		#endif // GENERATING_DOXYGEN_OUTPUT
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(VIEW_LEGACY::CREATEFLAGS)
DEFINE_ENUM_OP_BITWISE(VIEW_LEGACY::ICONVIEW)
#endif

//===================================================================================================================
//		Functions

//! Add 'recenter' tool.
void ToolAddRecenter (
	GRE::VIEW_LEGACY *view,
	GRE::TOOL **rtool = 0
	);

//! Add 'zoom box' tool.
void ToolAddZoomBox (
	GRE::VIEW_LEGACY *view,
	GRE::TOOL **rtool = 0
	);


//===================================================================================================================
}	// End namespace GRE

//! Add view-in-view tool.
void GRE_ToolAddViewInView (
	GRE::VIEW_LEGACY *view,
	GRE::TOOL **rtool = 0
	);

#endif	// INC_GRE_VIEWLEGACY_H

