/**
 * \file mgui/dlgcontactmi.h
 * \brief MGUI::DLG_CONTACTMI definition
 *
 * \if NODOC
 *
 * $Id: dlgcontactmi.h_v 1.48 2009/11/05 18:36:16 mju Exp $
 *
 * $Log: dlgcontactmi.h_v $
 * Revision 1.48  2009/11/05 18:36:16  mju
 * Change how custom controls are added to file selection dialogs.
 *
 * Revision 1.47  2009/09/02 17:53:57  bcolombe
 * Add options for setting to include settings, screenshot, session log
 *
 * Revision 1.46  2009/08/24 22:31:30  bcolombe
 * Make ListSetValue public
 *
 * Revision 1.45  2008/12/04 14:05:12  mju
 * Remove unused members, fix member/method order, exclude log from doxygen.
 *
 * Revision 1.44  2007/08/20 14:44:41  mju
 * Remove illegal qualifier.
 *
 * Revision 1.43  2007/05/25 14:36:23  honerkamp
 * adding opening an email client with attachment if send failed
 *
 * Revision 1.42  2007/05/08 12:48:25  mju
 * Use filepathlist hdr.
 *
 * Revision 1.40  2007/04/24 15:40:14  honerkamp
 * Changed the input reading from file_raw to file_text
 *
 * Revision 1.39  2007/03/30 15:15:35  honerkamp
 * added 4 new functions to set the fields of the dialog
 *
 * Revision 1.38  2007/03/30 14:48:57  honerkamp
 * added back in setdelegateaddfiles
 *
 * Revision 1.33  2007/03/15 19:44:15  honerkamp
 * Do not capture error screen shot in this class
 *
 * Revision 1.32  2007/03/08 23:10:59  honerkamp
 * Reorganized process to include the static function JustSaveReport
 *
 * Revision 1.31  2006/12/19 19:07:43  honerkamp
 * added a variable to tell whether the form was opened from an error message or a menu
 *
 * Revision 1.30  2006/12/06 00:29:31  HONERKAMP
 * added function to open the settings in a new window
 *
 * Revision 1.29  2006/10/23 21:02:48  HONERKAMP
 * added back in toggles when saving and loading
 *
 * Revision 1.17  2006/10/06 21:42:08  HONERKAMP
 * added functions to automaticaly delete temp movie files after send
 *
 * Revision 1.16  2006/10/05 21:45:15  HONERKAMP
 * Added buttons for capture movie and screen.
 *
 * Revision 1.15  2006/09/29 20:48:27  TURKER
 * added bool variables for savelinkonly and deleteaftersave toggle buttons
 *
 * Revision 1.14  2006/09/27 14:44:59  TURKER
 * Added functions to deal with save/load
 *
 * Revision 1.13  2006/09/25 16:07:59  TURKER
 * Added functions and variables for saving to file
 *
 * Revision 1.12  2006/06/05 19:43:02  ldyas
 * moved tempfile path to main object to enable deleting in destructor.
 *
 * Revision 1.3  2006/03/01 14:08:43  mju
 * Use correct inclusion guard name.
 * Put in mgui namespace.
 * Don't include headers we don't need.
 * Exclude private section from doxygen.
 *
 * Revision 1.0  2006/02/28 16:16:16  ldyas
 * \endif
**/

#ifndef INC_MGUI_DLGCONTACTMI_H
#define INC_MGUI_DLGCONTACTMI_H

#ifndef INC_MGUI_DLGSHELL_H
	#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_COMBOBOX_H
	#include <mgui/combobox.h>
#endif

#ifndef INC_MGUI_LISTBOX_H
	#include <mgui/listbox.h>
#endif

#ifndef INC_MGUI_GRID_H
	#include <mgui/grid.h>
#endif

#ifndef INC_MGUI_EDIT_H
	#include <mgui/edit.h>
#endif

#ifndef INC_MI32_FILETEXT_H
	#include <mi32/filetext.h>
#endif

#ifndef INC_MI32_FILEPATHLIST_H
	#include <mi32/filepathlist.h>
#endif

#ifndef INC_MI32_MDLG_H
	#include <mi32/mdlg.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class STATUSCONTEXT;
struct REPORTINFO;
#endif	// GENERATING_DOXYGEN_OUTPUT


namespace MGUI {

class DLG_CONTACTMI : public MGUI::DLGSHELL {
	public:

		//! Default constructor.
		DLG_CONTACTMI ();

		//! Construct with error details and message type
		DLG_CONTACTMI (
			const MISTRING& details,
			MDLG_MESSAGETYPE type = MDLG_MESSAGETYPE_Information
			); 

		//! Construct with message type.
		DLG_CONTACTMI (
			MDLG_MESSAGETYPE type
			);

		//! Destructor.		
		~DLG_CONTACTMI ();
		
		//! Set files list
		ERRVALUE ListSetValue (const FILEPATHLIST &PathList);

		//! Set the name field in the dialog.
		//! Should be called before DoModal
		void SetName(const MISTRING &name);

		//! Set the email field in the dialog.
		//! Should be called before DoModal
		void SetEmail(const MISTRING &email);

		//! Set the organization field in the dialog.
		//! Should be called before DoModal
		void SetOrganization(const MISTRING &organization);

		//! Set the subject field in the dialog.
		//! Should be called before DoModal
		void SetSubject(const MISTRING &subject);
		
		//! Sets a delegate that is called when the report is being processed and sent,
		//! It will add the additional files onto the report 
		//! Optional parameter to the delegate of deleting the file after sending
		void SetDelegateAddFiles (
			fastdelegate::FastDelegate<void (FILEPATHLIST & filelist)> fileadddelegate,
			bool deleteafteruse = false
			);

		//! Include user settings (tntproc, tnthost)
		void SetIncludeSettings (bool IncludeSettings);

		//! Include screenshot of last error
		void SetIncludeScreenshot (bool IncludeScreenshot);

		//! Include current session log
		void SetIncludeSessionLog (bool IncludeSessionLog);
		
		//! save the report without opening the contact support dialog
		static ERRVALUE JustSaveReport(
			MISTRING &details,
			MDLG_MESSAGETYPE type = MDLG_MESSAGETYPE_Information
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_CONTACTMI THISCLASS;
		
		MISTRING m_nameset;
		MISTRING m_emailset;
		MISTRING m_organizationset;
		MISTRING m_subjectset;
		
		MISTRING m_Details;
		MDLG_MESSAGETYPE m_Type;
		FILE_TEXT m_addfiles;
		FILE_TEXT m_loadfile;
		GENERICRW_TEXT::POSN m_filestart;
		const char *m_IniGroup;

		MGUI::CTRL_TOGGLEBUTTON m_LinkOnlyTB;
		MGUI::CTRL_TOGGLEBUTTON m_DeleteAfterSendTB;
		
		MGUI::LAYOUT_EQUALIZER m_equalizer;
		MGUI::LAYOUT_PANE m_subj;
		MGUI::CTRL_LABEL m_slab;
		MGUI::CTRL_LABEL m_desclab;
		MGUI::CTRL_GROUPBOX m_pane1;
		MGUI::LAYOUT_PANE m_misc2;
		MGUI::CTRL_LABEL m_desclab2;
		MGUI::LAYOUT_PANE m_misc3;
		MGUI::CTRL_LABEL m_desclab3;
		MGUI::LAYOUT_PANE m_misc4;
		MGUI::CTRL_LABEL m_desclab4;
		MGUI::CTRL_GROUPBOX m_togglepane;
		MGUI::LAYOUT_PANE m_misc5;
		MGUI::CTRL_GROUPBOX m_pane5;
		MGUI::LAYOUT_PANE m_BtnPane;
		MGUI::LAYOUT_PANE m_pane6;
		MGUI::LAYOUT_PANE m_pane7;

		MGUI::CTRL_EDIT_STRING m_Subject;
		MGUI::CTRL_EDIT_TEXT m_Description;
		MGUI::CTRL_TOGGLEBUTTON m_SendSettingsTB;
		MGUI::CTRL_TOGGLEBUTTON m_SendScreensTB;
		MGUI::CTRL_TOGGLEBUTTON m_SendLogTB;
		MGUI::CTRL_EDIT_STRING m_UserName;
		MGUI::CTRL_EDIT_STRING m_EmailAddress;
		MGUI::CTRL_EDIT_STRING m_Organization;
		MGUI::CTRL_GRID m_FileGrid;
		MGUI::CTRL_PUSHBUTTON m_ViewSettings;
		MGUI::CTRL_PUSHBUTTON m_ViewScreenShot;
		MGUI::CTRL_PUSHBUTTON m_ViewLog;
		MGUI::CTRL_PUSHBUTTON m_BtnSelect;
		MGUI::CTRL_PUSHBUTTON m_BtnRemove;
		MGUI::CTRL_PUSHBUTTON m_BtnRemoveAll;
		MGUI::CTRL_PUSHBUTTON m_PrivacyPolicyPB;
		MGUI::CTRL_PUSHBUTTON m_SaveToFilePB;
		MGUI::CTRL_PUSHBUTTON m_SendFromFilePB;
		MGUI::CTRL_PUSHBUTTON m_CreateMoviePB;
		MGUI::CTRL_PUSHBUTTON m_TakeScreenshotPB;
		MGUI::CTRL_PUSHBUTTON m_HelpPB;
		MGUI::CTRL_PUSHBUTTON m_SendPB;
		MGUI::CTRL_PUSHBUTTON m_SavePB;
		MGUI::CTRL_PUSHBUTTON m_CancelPB;

		bool m_bIncludeSettings;
		bool m_bIncludeScreenshot;
		bool m_bIncludeSessionLog;
		bool m_IsFromFile;  //changed to 1 after loading
		bool m_SaveLinkOnly;
		bool m_DeleteAfterSend;
		bool m_DirectoryListingIsCreated;//if the Directory Listing was created for viewing don't recreate it when sending
		bool m_SendErrorData; 
		bool m_DeleteExtraFiles; //delete the files that were added by the delegate after sending
		
		fastdelegate::FastDelegate<void (FILEPATHLIST &)> m_DelegateGetExtraFiles;
		FILEPATHLIST m_ExtraSendFiles;
		FILEPATH m_SavePath;
		FILEPATH m_SendPath;
		FILEPATH m_LoadPath;
		FILEPATH m_AddFilesPath; 
		FILEPATH m_SupportFolder;//used for loading not for saving
		FILEPATH m_DirectoryListing;
		FILEPATHLIST m_FilePathList;

		ERRVALUE MyWait(STATUSCONTEXT& StatusContext, int seconds);
		ERRVALUE DoSend ();
		ERRVALUE DoSave ();
		ERRVALUE DoLoad ();
		ERRVALUE SendEmail();

		//Creates the file either to be used as saving, or to be used to send
		//supports getting information both from the system and from the saved file
		ERRVALUE FillStructFromDialog(REPORTINFO &reportvariables);
		ERRVALUE ListSelect ();
		void ListClear ();
		void OnShowPolicy ();
		void OnViewHelp ();
		void OnSaveForm ();
		void OnFileSent ();
		void OnFileLoad ();
		void OnMovieCapture ();
		void OnScreenCapture ();
		void OnCaptureComplete ();
		void OnBtnRemove ();
		void OnBtnRemoveAll ();
		void OnBtnSelect ();
		void OnViewSettings();
		void OnViewScreenShot();
		void OnViewLog();
		void OnLinkOnly();
		void OnDeleteAfterSend();
		void OnSendFailed(MISTRING &reason);
		void ShowSettings(int index = 0);
		void SetFilesToAdd();
		void TestActive();
		void RemoveMovieFiles();
		int GetNextMovieNum();
		int GetNextScreenNum();
		int GetAllFiles(MISTRINGLIST &FileList, FILEPATH ParentFolder);
		bool ErrorShotIsFile();
		void CreateDeleteAfterSendControls (MGUI::LAYOUT_PANE_BASE& ParentPane);
		void CreateLinkOnlyControls (MGUI::LAYOUT_PANE_BASE& ParentPane);
		
		// DLGSHELL overrides
		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnOK ();
		virtual void v_OnCancel ();
		
		#endif	// GENERATING_DOXYGEN_OUTPUT
	};

}	// End namespace MGUI


#endif	// INC_MGUI_DLGCONTACTMI_H
