/**
 * \file mgui/dlggetobject.h
 * \brief Dialog functions to select files/objects
 *
 * \if NODOC
 * $Id: dlggetobject.h_v 1.6 2009/11/06 22:03:38 mju Exp $
 *
 * $Log: dlggetobject.h_v $
 * Revision 1.6  2009/11/06 22:03:38  mju
 * Use const with filter so can use implicit ctors.
 *
 * Revision 1.5  2009/11/06 14:04:03  mju
 * Move dlg_getobject_style enum to own hdr.
 *
 * Revision 1.4  2009/11/02 21:13:30  mju
 * Add style_allowChooseTextEncodiing.
 *
 * Revision 1.3  2009/10/26 20:51:58  mju
 * Add dlgGetFile...
 *
 * Revision 1.2  2009/10/22 21:54:30  mju
 * Add dlgFileManager.
 *
 * Revision 1.1  2009/10/22 21:50:02  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGGETOBJECT_H
#define	INC_MGUI_DLGGETOBJECT_H

#ifndef	INC_MGUI_DLGGETOBJECTSTYLE_H
	#include <mgui/dlggetobjectstyle.h>
#endif

#ifndef	INC_MGUI_DLGGETOBJECTFILTER_H
	#include <mgui/dlggetobjectfilter.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class FILEPATHLIST;
#endif

namespace MGUI {

//=====================================================================================================================

//! Dialog for "file manager".
void DlgFileManager (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	bool modal
	);


//! Dialog to select a single file.
ERRVALUE DlgGetFile (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& title,						//!< Title for dialog
	const DLG_GETOBJECT_FILTER& filter,		//!< Filter to determine allowed files
	FILEPATH& filepath,							//!< Selected file passed/returned
	DLG_GETOBJECT_STYLE dlgstyle = DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Dialog style
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);

//! Dialog to select variable list of files.
ERRVALUE DlgGetFileList (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& title,						//!< Title for dialog
	const DLG_GETOBJECT_FILTER& filter,		//!< Filter to determine allowed files
	FILEPATHLIST& filepathlist,				//!< File list returned
	DLG_GETOBJECT_STYLE dlgstyle = DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Dialog style
	int minobjs = 1,								//!< Minimum number of files required
	int maxobjs = 0,								//!< Maximum allowed to be selected, 0 if no limit
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);

//! Dialog to select a specific set of files, TEXTID labels.
ERRVALUE DlgGetFileSet (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& title,						//!< Title for dialog
	const DLG_GETOBJECT_FILTER& filter,		//!< Filter to determine allowed files
	const TEXTID labelids[],					//!< Label IDs for each file
	FILEPATHLIST& filepathlist,				//!< File list passed/returned
	DLG_GETOBJECT_STYLE dlgstyle = DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Dialog style
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);

//! Dialog to select a specific set of files, MISTRING labels.
ERRVALUE DlgGetFileSet (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& title,						//!< Title for dialog
	const DLG_GETOBJECT_FILTER& filter,		//!< Filter to determine allowed files
	const MISTRING labelstr[],					//!< Label names for each file
	FILEPATHLIST& filepathlist,				//!< File list passed/returned
	DLG_GETOBJECT_STYLE dlgstyle = DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Dialog style
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);

//! Dialog to select a specific set of files, labels derived from file list provided.
ERRVALUE DlgGetFileSet (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& title,						//!< Title for dialog
	const DLG_GETOBJECT_FILTER& filter,		//!< Filter to determine allowed files
	FILEPATHLIST& filepathlist,				//!< File list passed/returned
	DLG_GETOBJECT_STYLE dlgstyle = DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Dialog style
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);


//! Dialog to select a single object.
ERRVALUE DlgGetObject (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& title,						//!< Title for dialog
	const DLG_GETOBJECT_FILTER& filter,		//!< Filter to determine allowed objects
	RVC::OBJITEM& objitem,						//!< Selected object passed/returned
	DLG_GETOBJECT_STYLE dlgstyle = DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Dialog style
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);

//! Dialog to select child object, with no navigation permitted.
//! Also automatically allows selection of table if parent is a database and only OBJTYPE_Table specified.
//! If the specified 'objitem' is not a child of 'parentobjitem' then it will be ignored.
ERRVALUE DlgGetObjectChild (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const RVC::OBJITEM& ObjItemParent,		//!< Parent object location, must exist
	const MISTRING& title,						//!< Title for dialog
	const DLG_GETOBJECT_FILTER& filter,		//!< Filter to determine allowed objects
	RVC::OBJITEM& ObjItemChild,				//!< Selected child location passed/returned
	DLG_GETOBJECT_STYLE dlgstyle = DLG_GETOBJECT_STYLE_OnlyExisting	//!< Dialog style
	);

//! Dialog to select variable list of objects.
ERRVALUE DlgGetObjectList (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& title,						//!< Title for dialog
	const DLG_GETOBJECT_FILTER& filter,		//!< Filter to determine allowed objects
	RVC::OBJITEMLIST& objlist,					//!< Object list returned
	DLG_GETOBJECT_STYLE dlgstyle = DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Dialog style
	int minobjs = 1,								//!< Minimum number of objects required
	int maxobjs = 0,								//!< Maximum allowed to be selected, 0 if no limit
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);

//! Dialog to select a specific set of objects, TEXTID labels.
ERRVALUE DlgGetObjectSet (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& title,						//!< Title for dialog
	const DLG_GETOBJECT_FILTER& filter,		//!< Filter to determine allowed objects
	const TEXTID labelids[],					//!< Label IDs for each object
	RVC::OBJITEMLIST& objlist,					//!< Object list passed/returned
	DLG_GETOBJECT_STYLE dlgstyle = DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Dialog style
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);

//! Dialog to select a specific set of objects, MISTRING labels.
ERRVALUE DlgGetObjectSet (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& title,						//!< Title for dialog
	const DLG_GETOBJECT_FILTER& filter,		//!< Filter to determine allowed objects
	const MISTRING labelstr[],					//!< Label names for each object
	RVC::OBJITEMLIST& objlist,					//!< Object list passed/returned
	DLG_GETOBJECT_STYLE dlgstyle = DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Dialog style
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);

//! Dialog to select a specific set of objects, labels derived from object list provided.
ERRVALUE DlgGetObjectSet (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& title,						//!< Title for dialog
	const DLG_GETOBJECT_FILTER& filter,		//!< Filter to determine allowed objects
	RVC::OBJITEMLIST& objlist,					//!< Object list passed/returned
	DLG_GETOBJECT_STYLE dlgstyle = DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Dialog style
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_DLGGETOBJECT_H
