/**
 * \file mgui/dlggetobjectstyle.h
 * \brief MGUI::DLG_GETOBJECT_STYLE enumeration definition.
 *
 * \if NODOC
 * $Id: dlggetobjectstyle.h_v 1.5 2010/08/15 23:53:38 bcolombe Exp $
 *
 * $Log: dlggetobjectstyle.h_v $
 * Revision 1.5  2010/08/15 23:53:38  bcolombe
 * Added DefaultToWeblayers and ShowProprietaryWebLayers
 *
 * Revision 1.4  2010/08/11 21:50:30  bcolombe
 * Added WebLayers style
 *
 * Revision 1.3  2009/11/24 16:03:08  mju
 * Add noShowFileTypes.
 *
 * Revision 1.2  2009/11/23 16:25:29  mju
 * Add noShowObjTypes.
 *
 * Revision 1.1  2009/11/06 14:03:52  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGGETOBJECTSTYLE_H
#define	INC_MGUI_DLGGETOBJECTSTYLE_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Enumeration of object selection dialog style flags.
enum DLG_GETOBJECT_STYLE {
	DLG_GETOBJECT_STYLE_OnlyExisting =						0x00000000,	//!< Only allow existing items
	DLG_GETOBJECT_STYLE_OnlyNew =								0x00000001,	//!< Only allow new items, don't let user select existing
	DLG_GETOBJECT_STYLE_AllowDuplicate =					0x00000010,	//!< Allow duplicate objects in DLG_GETOBJECT_LIST
	DLG_GETOBJECT_STYLE_AllowFewer =							0x00000020,	//!< Allow fewer than requested number of objects in RVC::DlgGetObjectSet()
	DLG_GETOBJECT_STYLE_AllowLinkForNew =					0x00000040,	//!< Allow "linkable" file types even if only creating new object
	DLG_GETOBJECT_STYLE_AllowNewAndExisting =				0x00000080,	//!< Allow both new and existing item selection, cannot combine with DLG_GETOBJECT_STYLE_OnlyNew
	DLG_GETOBJECT_STYLE_AllowNone =							0x00000100,	//!< Allow choice of "none", will return R_SkipRequest if selected
	DLG_GETOBJECT_STYLE_AllowSkip =							0x00000200,	//!< Allow user to skip this selection
	DLG_GETOBJECT_STYLE_AllowBrowseIfSelectableChild =	0x00000400,	//!< Allow browse into non-folder object if has selectable child/descendent
	DLG_GETOBJECT_STYLE_AllowFolder =						0x00000800,	//!< Allow to select "folder" in addition to other allowed items
	DLG_GETOBJECT_STYLE_NoAutoName =							0x00001000,	//!< Disallow "auto-name" for DLG_GETOBJECT_SET
	DLG_GETOBJECT_STYLE_NoMetaData =							0x00002000,	//!< Don't allow meta-data viewing/selection
	DLG_GETOBJECT_STYLE_NoNavigate =							0x00004000,	//!< User can't navigate to new location
	DLG_GETOBJECT_STYLE_NoWarnOverwrite =					0x00008000,	//!< Don't warn user that existing object will be overwritten, use with DLG_GETOBJECT_STYLE_AllowNewAndExisting
	DLG_GETOBJECT_STYLE_AllowChooseTextEncoding =		0x00010000,	//!< Include control to choose text character set encoding
	DLG_GETOBJECT_STYLE_NoShowObjTypes =					0x00020000,	//!< Don't include choice of which object types to show
	DLG_GETOBJECT_STYLE_NoShowFileTypes =					0x00040000,	//!< Don't include choice of which file types to show
	DLG_GETOBJECT_STYLE_WebLayers =							0x00080000,	//!< Allow web location as items
	DLG_GETOBJECT_STYLE_DefaultToWebLayers =				0x00100000,	//!< Default web layers as browse location
	DLG_GETOBJECT_STYLE_ShowProprietaryWebLayers =			0x00200000,	//!< Show google maps/bing maps base layers
#ifndef GENERATING_DOXYGEN_OUTPUT
	DLG_GETOBJECT_STYLE_AllowMultiSelect =					0x80000000	// Used internally to allow multiple selection in browse and catalog lists
#endif
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DLG_GETOBJECT_STYLE)
#endif

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_DLGGETOBJECTSTYLE_H
