/**
 * \file mgui/dlgquestion.h
 * \brief MGUI::DLG_QUESTION class definitions
 *
 * \if NODOC
 * $Id: dlgquestion.h_v 1.11 2011/11/21 21:28:02 mju Exp $
 *
 * $Log: dlgquestion.h_v $
 * Revision 1.11  2011/11/21 21:28:02  mju
 * Remove redundant hdr.
 *
 * Revision 1.10  2011/08/25 22:18:47  mju
 * Use pushbutton::style_WideMargins.
 *
 * Revision 1.9  2006/01/10 15:12:29  mju
 * No char* for labels.
 *
 * Revision 1.8  2005/06/24 14:08:50  mju
 * No template for togglebutton.
 *
 * Revision 1.7  2004/11/19 16:30:54  mju
 * Add ability to add non-standard buttons.
 * Use INT32 for response values so works with non-std buttons.
 *
 * Revision 1.6  2003/10/03 19:58:19  linux32build!build
 * Doxygen
 *
 * Revision 1.2  2003/09/08 13:32:29  mju
 * Fix virtual override name.
 *
 * Revision 1.1  2003/09/05 21:26:50  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGQUESTION_H
#define	INC_MGUI_DLGQUESTION_H

#ifndef	INC_MI32_MDLG_H
#include <mi32/mdlg.h>
#endif

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_LISTITEM_H
#include <mgui/listitem.h>
#endif


namespace MGUI {

//! Dialog to display question to user with pushbutton responses.
//! Also allows for optional toggle to always show / never show.
//! @see MdlgQuestion.
class DLG_QUESTION : public MGUI::DLGSHELL {
	public:

		enum TOGGLETYPE {
			TOGGLETYPE_NeverAsk,				//!< Toggle "on" to never ask again, default is "off"
			TOGGLETYPE_AlwaysAsk				//!< Toggle "on" to always ask, default is "on"
			};

		//! Constructor.
		DLG_QUESTION (
			const MISTRING& dlgtitle,				//!< Text for title
			const MISTRING& text,					//!< Text for question
			DLGQ_STYLE style,							//!< Dialog style
			INT32 dftresponse,						//!< Default response value
			HELPID helpid = HELPID__None			//!< Help ID
			);

		//! Destructor.
		~DLG_QUESTION (
			);

		//! Add non-standard button with text and response.
		//! Button will be added to left of any "standard" buttons used.
		void AddButton (
			const MISTRING& text,
			INT32 response
			);

		//! Determine if user doesn't want to see question again.
		//! @return true if user does not want to see question, false if does.
		bool CheckNoShow (
			) const;

		//! Get response from dialog.
		INT32 GetResponse (
			) const { return (m_response); }

		//! Set up toggle for whether user wants to see question again.
		//! @return true if user does not want to see question, false if does.
		bool SetupToggle (
			const char *IniGroup,					//!< Ini group
			const char *IniName,						//!< Ini name to store toggle state
			const MISTRING& ToggleText,			//!< Text to show for toggle
			TOGGLETYPE ToggleType					//!< Meaning of toggle
			);

	private:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class BUTTON : public MGUI::CTRL_PUSHBUTTON {
			public:
				BUTTON (
					DLG_QUESTION *pDlg,
					INT32 response
					);
				BUTTON (
					const BUTTON& rhs
					);
				void Create (
					MGUI::LAYOUT_PANE_BASE& ParentPane,
					const MISTRING& label
					) { CTRL_PUSHBUTTON::Create(ParentPane,label,MGUI::CTRL_PUSHBUTTON::STYLE_WideMargins); }
			private:
				DLG_QUESTION *m_pDlg;
				INT32 m_response;
				virtual void OnPressed ();
			};

		INT32 m_response;
		MISTRING m_text;
		DLGQ_STYLE m_style;
		MILIST<LISTITEM> m_ExtraButtonItems;
		char *m_ToggleIniGroup;
		char *m_ToggleIniName;
		MISTRING m_ToggleText;
		TOGGLETYPE m_ToggleType;
		CTRL_LABEL m_TextLabel;
		CTRL_TOGGLEBUTTON m_Toggle;
		MILIST<BUTTON> m_Buttons;

		BUTTON& CreateButtonLow (
			TEXTID textid,
			DLGRESPONSE response
			);

		void OnToggle ();

		void SetResponse (
			INT32 response
			) { m_response = response; }

		//! DLGSHELL overrides.
		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnCancel ();
		virtual void v_OnOK ();

		friend class DLG_QUESTION::BUTTON;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


}	// End namespace MGUI


#endif	// INC_MGUI_DLGQUESTION_H
