/*****************************************************************************
 * \file mgui/xmldlg.h
 * \brief XML Dialog
 *	
 * \if NODOC
 * $Log: xmldlg.h_v $
 * Revision 1.12  2011/12/02 22:33:12  mju
 * Remove redundant hdrs.
 *
 * Revision 1.11  2007/05/09 16:03:01  dwilliss
 * SML namepace issues
 *
 * Revision 1.10  2006/05/25 19:47:31  dwilliss
 * Added v_OnCloseRequest
 *
 * Revision 1.7  2005/02/18 16:39:28  dwilliss
 * XML_DLG is now derived from DLGSHELL, not older deprecated class
 *
 * Revision 1.1  2002/09/03 22:38:57  dwilliss
 * Initial revision
 * \endif
 ****************************************************************************/

#ifndef INC_MGUI_XMLDLG_H
#define INC_MGUI_XMLDLG_H


#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_XMLDATA_H
#include <mgui/xmldata.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace SML {
	class CONTEXT;
	}
namespace MGUI {
	class CTRL_LABEL;
	class CTRL_GROUPBOX;
	class CTRL_PUSHBUTTON;
	class CTRL_TOGGLEBUTTON;
	class CTRL_EDIT_STRING;
	class CTRL_EDIT_NUMBER;
	class FORM_EDIT_RANGE;
	class GUIXMLPRIVDATA;
	}
#endif


namespace MGUI {

class LAYOUT_PANE_XMLFORM : public MGUI::LAYOUT_PANE {
	public:
		LAYOUT_PANE_XMLFORM();
		virtual ~LAYOUT_PANE_XMLFORM();

		//! Create a layout from a \<form\> in an XML document

		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const XMLNODE* xmlnode,						//!< XML document node describing the form
			SML::CONTEXT* context = 0						//!< Context to use if being created from SML
			);

		//! Get a control by it's XML id attribute.
		//! This version just returns a pointer to an MGUI::FORM.  This should
		//! only be used in cases where you don't really care what the control
		//! type is.  If you expect the control to be of a specific type, use
		//! the override that takes a referenct to a pointer.
		//! @return NULL if there was no such control.
		MGUI::FORM* GetCtrlByID (
			const MIUNICODE* id
			) const;
		
		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::CTRL_LABEL*& ctrl
			) const;

		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::CTRL_GROUPBOX*& ctrl
			) const;

		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::CTRL_PUSHBUTTON*& ctrl
			) const;

		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::CTRL_TOGGLEBUTTON*& ctrl
			) const;

		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::CTRL_EDIT_STRING*& ctrl
			) const;

		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::CTRL_EDIT_NUMBER*& ctrl
			) const;

		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::FORM_EDIT_RANGE*& ctrl
			) const;


		//! Get a pane by it's XML id attribute.
		//! @return NULL if there was no such pane.
		MGUI::LAYOUT_PANE_BASE* GetPaneByID (
			const MIUNICODE* id
			) const;
		
		//! Get the current values of the controls.
		//! Controls with no id attribute are ignored
		ERRVALUE GetValues (
			MGUI::XMLFORM_DATA& values
			) const;

		//! Get the current values of the controls.
		//! Controls with no id attribute are ignored
		ERRVALUE SetValues (
			const MGUI::XMLFORM_DATA& values
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		GUIXMLPRIVDATA *m_priv;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


class DLG_XML : public MGUI::DLGSHELL {
	public:
		DLG_XML();
		virtual ~DLG_XML();

		//! Get a control by it's XML id attribute.
		//! This version just returns a pointer to an MGUI::FORM.  This should
		//! only be used in cases where you don't really care what the control
		//! type is.  If you expect the control to be of a specific type, use
		//! the override that takes a referenct to a pointer.
		//! @return NULL if there was no such control.
		MGUI::FORM* GetCtrlByID (
			const MIUNICODE* id
			) const;
		
		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::CTRL_LABEL*& ctrl
			) const;

		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::CTRL_GROUPBOX*& ctrl
			) const;

		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::CTRL_PUSHBUTTON*& ctrl
			) const;

		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::CTRL_TOGGLEBUTTON*& ctrl
			) const;

		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::CTRL_EDIT_STRING*& ctrl
			) const;

		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::CTRL_EDIT_NUMBER*& ctrl
			) const;

		//! Get a control by it's XML id attribute.
		//! Returns true if there was such a control, false if not.
		//! Also returns false if there was a control with the given id
		//! but was not of the requested type.
		bool GetCtrlByID (
			const MIUNICODE* id,
			MGUI::FORM_EDIT_RANGE*& ctrl
			) const;


		//! Get a pane by it's XML id attribute.
		//! @return NULL if there was no such pane.
		MGUI::LAYOUT_PANE_BASE* GetPaneByID (
			const MIUNICODE* id
			) const;
		
		//! Get the current values of the controls.
		//! Controls with no id attribute are ignored
		ERRVALUE GetValues (
			MGUI::XMLFORM_DATA& values
			) const;

		//! Get the current values of the controls.
		//! Controls with no id attribute are ignored
		ERRVALUE SetValues (
			const MGUI::XMLFORM_DATA& values
			);


		//! Create a layout from a \<form\> in an XML document
		ERRVALUE SetXMLNode (
			const XMLNODE* xmlnode,						//!< XML document node describing the dialog
			SML::CONTEXT* context = 0						//!< Context to use if being created from SML
			);


	protected:

		virtual ERRVALUE v_CreateContent ();			//! Creates the body from the form contents
		virtual void v_OnApply ();
		virtual void v_OnOK ();
		virtual void v_OnCancel ();
		virtual void v_OnOpen ();
		virtual void v_OnClose ();
		virtual void v_OnCloseRequest ();
		virtual void v_OnDestroy ();

		GUIXMLPRIVDATA* GetPriv() const { return (m_priv); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		GUIXMLPRIVDATA *m_priv;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


}		// End of MGUI namespace

#endif

