/**
 * \file mi32/autoretryfileraw.h
 * \brief AUTORETRY_FILE_RAW definitions
 *
 * \if NODOC
 * $Id: autoretryfileraw.h_v 1.1 2012/08/29 15:53:51 mju Exp $
 *
 * $Log: autoretryfileraw.h_v $
 * Revision 1.1  2012/08/29 15:53:51  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_AUTORETRYFILERAW_H
#define	INC_MI32_AUTORETRYFILERAW_H

#ifndef	INC_MI32_AUTORETRY_H
#include <mi32/autoretry.h>
#endif

#ifndef	INC_MI32_FILERAW_H
#include <mi32/fileraw.h>
#endif

#ifndef	INC_MI32_DELEGATE_H
	#include <mi32/delegate.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

//! Support for automatic retry of FILE_RAW operation.
class CLASSLIBEXPORT AUTORETRY_FILE_RAW : public AUTORETRY {
	public:

		//! Delegate to retry "Open".
		typedef fastdelegate::FastDelegate<ERRVALUE(const AUTORETRY_FILE_RAW&, const FILEPATH&, FILE_ACCESS, FILE_SHARE)> DELEGATE_RETRY_OPEN;

		//! Register delegate to retry "Open" in interactive mode.
		static void RegisterDelegateRetryOpenInteractive (
			DELEGATE_RETRY_OPEN delegate
			);

		//! Construct with FILE_RAW reference.
		explicit AUTORETRY_FILE_RAW (
			FILE_RAW& file						//!< Instance to attach
			);

		//! Destructor.
		~AUTORETRY_FILE_RAW ();

		FILE_RAW& AccessFileRaw () const
			{ return (m_file); }

		//! Open referenced file.
		ERRVALUE Open (
			const FILEPATH& filepath,
			FILE_ACCESS access,
			FILE_SHARE share = FILE_SHARE_Auto
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FILE_RAW& m_file;

		AUTORETRY_FILE_RAW (const AUTORETRY_FILE_RAW&);
		AUTORETRY_FILE_RAW& operator= (const AUTORETRY_FILE_RAW&);
		#endif
	};

#undef	CLASSLIBEXPORT
#endif	// INC_MI32_AUTORETRYFILERAW_H
