/**
 * \file mi32/rvcdatetime.h
 * \brief RVCDATETIME structure definition
 *
 * \if NODOC
 * $Id: rvcdatetime.h_v 1.1 2007/09/06 22:34:28 scowan Exp $
 *
 * $Log: rvcdatetime.h_v $
 * Revision 1.1  2007/09/06 22:34:28  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_RVCDATETIME_H
#define	INC_MI32_RVCDATETIME_H

#ifndef INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif

#include <time.h>

struct RVCDATETIME {
	UINT8 year;									//!<  Add 1900 to get actual year 
	UINT8 month;								//!<  1-12 
	UINT8 day;									//!<  1-31 
	UINT8 hour;									//!<  0-23 
	UINT8 minute;								//!<  0-59 
	UINT8 second;								//!<  0-59 
	UINT16 count;								//!<  count; if more than one object created with this time 
	
	int Compare (
		const RVCDATETIME& rhs
		) const {
		if (this->year < rhs.year) return (1);
		else if (this->year > rhs.year) return (-1);
		if (this->month < rhs.month) return (1);
		else if (this->month > rhs.month) return (-1);
		if (this->day < rhs.day) return (1);
		else if (this->day > rhs.day) return (-1);
		if (this->hour < rhs.hour) return (1);
		else if (this->hour > rhs.hour) return (-1);
		if (this->minute < rhs.minute) return (1);
		else if (this->minute > rhs.minute) return (-1);
		if (this->second < rhs.second) return (1);
		else if (this->second > rhs.second) return (-1);
		return (0);
		}
		
	void SetCurrentTime (
		) {
		time_t curtime;
		time(&curtime);
		SetTime(curtime);
		return;
		}
	
	void SetTime (
		const time_t& curtime
		) {
		struct tm *tm = localtime(&curtime);
		this->year = static_cast<UINT8>(tm->tm_year);
		this->month = static_cast<UINT8>(tm->tm_mon + 1);
		this->day = static_cast<UINT8>(tm->tm_mday);
		this->hour = static_cast<UINT8>(tm->tm_hour);
		this->minute = static_cast<UINT8>(tm->tm_min);
		this->second = static_cast<UINT8>(tm->tm_sec);
		this->count = 0;
		return;
		}
		
	};

inline bool operator== (
	const RVCDATETIME& lhs,
	const RVCDATETIME& rhs
	) { return (lhs.Compare(rhs) == 0); }
	
inline bool operator!= (
	const RVCDATETIME& lhs,
	const RVCDATETIME& rhs
	) { return (lhs.Compare(rhs) != 0); }
	
inline bool operator< (
	const RVCDATETIME& lhs,
	const RVCDATETIME& rhs
	) { return (lhs.Compare(rhs) == 1); }
	
inline bool operator<= (
	const RVCDATETIME& lhs,
	const RVCDATETIME& rhs
	) { return (lhs.Compare(rhs) != -1); }
	
inline bool operator> (
	const RVCDATETIME& lhs,
	const RVCDATETIME& rhs
	) { return (lhs.Compare(rhs) == -1); }

inline bool operator>= (
	const RVCDATETIME& lhs,
	const RVCDATETIME& rhs
	) { return (lhs.Compare(rhs) != 1); }

#endif	// INC_MI32_RVCGENINFO_H
