/**
 * \file <mi32/statsimple.h>
 * \brief STAT_SIMPLE class definitions.
 *
 * \if NODOC
 * $Id: statsimple.h_v 1.3 2012/04/30 14:45:51 mju Exp $
 *
 * $Log: statsimple.h_v $
 * Revision 1.3  2012/04/30 14:45:51  mju
 * Add first/last values.
 *
 * Revision 1.2  2012/04/23 17:45:56  mju
 * Change to use weight instead of count.
 *
 * Revision 1.1  2012/04/23 17:00:47  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_STATSIMPLE_H
#define	INC_MI32_STATSIMPLE_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

//=====================================================================================================================

//! Container for simple statistics for single variable.
//! Statistics include minimum, maximum, sum, total weight, with methods to compute mean, variance.

class CLASSLIBEXPORT STAT_SIMPLE {
	public:

		//! Default constructor.
		STAT_SIMPLE ();

		//! Destructor.
		~STAT_SIMPLE ();

		//! Add value with optional weight.
		void Add (
			double value,
			double weight = 1		//!< Weight for this value, must be positive
			);

		//! Clear statistics.
		void Clear ();

		//! Compute mean per unit weight.
		double ComputeMean () const;

		//! Compute standard deviation.
		double ComputeStandardDeviation () const;

		//! Get first value added.
		double GetFirst () const
			{ return (m_First); }

		//! Get last value added.
		double GetLast () const
			{ return (m_Last); }

		//! Get maximum value.
		double GetMaximum () const
			{ return (m_Maximum); }

		//! Get minimum value.
		double GetMinimum () const
			{ return (m_Minimum); }

		//! Get weighted sum of all values added.
		double GetSum () const
			{ return (m_Sum); }

		//! Get total weight of values added.
		double GetTotalWeight () const
			{ return (m_TotalWeight); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		double m_Minimum;
		double m_Maximum;
		double m_First;
		double m_Last;
		double m_Sum;
		double m_Sum2;
		double m_TotalWeight;
		#endif
	};

//=====================================================================================================================


#undef CLASSLIBEXPORT

#endif	// INC_MI32_STATSIMPLE_H
