/**
 * \file mi32/stipplemask.h
 * \brief STIPPLEMASK definitions.
 *
 * \if NODOC
 * $Id: stipplemask.h_v 1.2 2012/09/18 13:50:38 mju Exp $
 *
 * $Log: stipplemask.h_v $
 * Revision 1.2  2012/09/18 13:50:38  mju
 * Dox ctor fix.
 *
 * Revision 1.1  2011/12/19 15:48:17  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_STIPPLEMASK_H
#define	INC_MI32_STIPPLEMASK_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif


class STIPPLEMASK {
	public:

		//! Default constructor, creates empty stipple.
		STIPPLEMASK (
			) : m_Data(0), m_SizeX(0), m_SizeY(0) { }

		//! Copy constructor.
		STIPPLEMASK (const STIPPLEMASK& rhs);

		//! Destructor.
		~STIPPLEMASK ()
			{ delete [] m_Data; }

		//! Assignment.
		STIPPLEMASK& operator= (const STIPPLEMASK& rhs);

		//! Clear stipple.
		void Clear ();

		//! Get pointer to start of data array.
		const bool* GetData () const
			{ return (m_Data); }

		//! Get pointer to data array row for specified Y value.
		const bool* GetDataY (
			INT32 y
			) const {
			return (m_Data + (y % m_SizeY) * m_SizeX);
			}

		//! Get X size (width) of mask.
		UINT16 GetSizeX () const
			{ return (m_SizeX); }

		//! Get Y size (width) of mask.
		UINT16 GetSizeY () const
			{ return (m_SizeY); }

		//! Determine if stipple is defined.
		bool IsDefined () const
			{ return (m_Data != 0); }

		//! Set stipple values from bool array.
		void Set (
			UINT16 sizeX,				//!< X size
			UINT16 sizeY,				//!< Y size
			const bool *mask			//!< Stipple mask
			);

		//! Set stipple values from UINT8 array.
		void Set (
			UINT16 sizeX,				//!< X size
			UINT16 sizeY,				//!< Y size
			const UINT8 *mask			//!< Stipple mask
			);

		//! Test if point is included by stipple.
		bool TestPoint (
			INT32 x,						//!< Point X coordinate
			INT32 y						//!< Point Y coordinate
			) const { return (m_Data[(y % m_SizeY) * m_SizeX + (x % m_SizeX)]); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		bool *m_Data;
		UINT16 m_SizeX;
		UINT16 m_SizeY;
		#endif
	};

#endif	// INC_MI32_STIPPLEMASK_H
