/**
 * \file textres.h <mi32/textres.h>
 * \brief TEXTRES definitions
 *
 * \if NODOC
 * $Id: textres.h_v 1.14 2006/10/06 17:39:39 mju Exp $
 *
 * $Log: textres.h_v $
 * Revision 1.14  2006/10/06 17:39:39  mju
 * Move textresitem/list to own hdrs.
 *
 * Revision 1.13  2005/09/08 20:24:14  mju
 * Add allowreplace parm to initialize.
 *
 * Revision 1.12  2004/10/25 19:49:56  mju
 * Add getchk().
 *
 * Revision 1.11  2004/04/01 20:42:18  mju
 * Add nameset_nolookup.
 * Add clear() method.
 * Add assign method using UINT8* for utf8.
 *
 * Revision 1.10  2003/10/09 19:48:41  mju
 * Add ValidateFile.
 *
 * Revision 1.9  2003/09/26 15:00:26  mju
 * Add textresitem::isvalid.
 * Change comment for CreateFromXML to reference 'lookup' property.
 *
 * Revision 1.8  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.7  2003/09/11 13:57:26  mju
 * Add DLL check for textresitem::createfromxml.
 *
 * Revision 1.6  2003/09/10 16:36:16  mju
 * Add GetString variant taking parm to specify whether want name if not found.
 *
 * Revision 1.5  2003/09/05 13:10:16  mju
 * Have textresitem::getstring return const reference.
 *
 * Revision 1.4  2003/09/03 20:20:17  mju
 * Add operator< and == so can export milist.
 *
 * Revision 1.3  2003/09/03 16:39:26  mju
 * Add textresitem::CreateFromXML.
 *
 * Revision 1.2  2003/08/26 15:40:23  mju
 * Don't include AddFile method for static library as don't want to require XML.
 *
 * Revision 1.1  2003/08/22 20:44:06  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_TEXTRES_H
#define	INC_MI32_TEXTRES_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class MISTRING;
class FILEPATH;
struct CHECKEXE;
#ifndef NODLLLINKAGE
class XMLNODE;
#endif
#endif // GENERATING_DOXYGEN_OUTPUT

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif


//! Text resource nameset enumeration.
enum TEXTRES_NAMESET {
	TEXTRES_NAMESET_General = 	0,		//!< General text
	TEXTRES_NAMESET_Label =		1,		//!< Button, menu, etc user interface control label
	TEXTRES_NAMESET_Title =		2,		//!< Window title
	TEXTRES_NAMESET_Help =		3,		//!< Help text
	TEXTRES_NAMESET_COUNT,
	TEXTRES_NAMESET_NoLookup = TEXTRES_NAMESET_COUNT
	};


//! Class for text resource lookup.
class CLASSLIBEXPORT TEXTRES {
	public:

	#ifndef NODLLLINKAGE
		//! Add specified file to text resource set.
		// Not supported in static library.
		static ERRVALUE AddFile (
			const FILEPATH& filepath					//!< Path to file, must be in proper XML or binary format
			);
	#endif

		//! Get text string using number (TEXTID, HELPID or ERRVALUE).
		//! If number has no associated text returned string will be blank if number is positive
		//! or "Unknown error [number]" if negative.
		//! @return String.
		static MISTRING GetString (
			int num											//!< TEXTID, HELPID or ERRVALUE
			);

		//! Get text string using name.
		//! @return String, will be "Unknown text item 'name'" if not found.
		static MISTRING GetString (
			const char *name,								//!< Item name
			TEXTRES_NAMESET nameset = TEXTRES_NAMESET_General
			);

		//! Get text string using name.
		//! @return String.
		static MISTRING GetString (
			const char *name,								//!< Item name
			TEXTRES_NAMESET nameset,
			bool UseNameIfNotFound						//!< Return specified name if not found
			);

		//! Determine if entry exists for specified name.
		//! @return true if entry exists, false if not.
		static bool HasEntry (
			const char *name,								//!< Item name
			TEXTRES_NAMESET nameset = TEXTRES_NAMESET_General
			);

	#ifdef MISYSTEMDLL
		//! Initialize text resource system.
		//! Normally called by application startup code.
		//! Must be called before using other TEXTRES methods.
		//! @return true if successful, false if not.
		static bool Initialize (
			);
	#endif

		//! Initialize text resource system.
		//! Normally called by application startup code.
		//! Must be called before using other TEXTRES methods.
		//! @return true if successful, false if not.
		static bool Initialize (
			bool AllowReplace = false					//!< Allow file to be replaced while process running
			);

		//! Load X resources.
		static ERRVALUE LoadX (
			void (*PutResFunc)(void*,const char*),	//!< Function to call to 'put' single X resource line (usually XrmPutLineResource)
			void *PutResData								//!< Data to pass to function (usually pointer to X resource database)
			);

		//! Terminate text resource system.
		//! Normally called by application termination code.
		static void Terminate (
			);

	#ifndef NODLLLINKAGE
		//! Validate specified file.
		// Not supported in static library.
		static ERRVALUE ValidateFile (
			const FILEPATH& filepath					//!< Path to file
			);
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
		static const CHECKEXE& GetChk ();
	#endif

	};


#undef	CLASSLIBEXPORT

#endif	// INC_MI32_TEXTRES_H
