/**
 * \file rvc/opslantground.h
 * \brief Definitions for RVC::OP::SLANTGROUND class.
 *
 * \if NODOC
 * $Id: opslantground.h_v 1.3 2007/02/01 20:17:57 bcolombe Exp $
 *
 * $Log: opslantground.h_v $
 * Revision 1.3  2007/02/01 20:17:57  bcolombe
 * Capitalize Look in Angle methods
 *
 * Revision 1.2  2007/02/01 20:14:27  bcolombe
 * Changed to GetSurfaceElevation
 *
 * Revision 1.1  2007/02/01 15:50:24  bcolombe
 * Initial revision
 *
 *
 * \endif
**/

#ifndef	INC_RVC_OPSLANTGROUND_H
#define	INC_RVC_OPSLANTGROUND_H

#ifndef  INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
class RASTER;
class OBJITEM;
#endif

namespace OP {

//! Class to compute the SLANT to GROUND conversion of radar images
class SLANTGROUND {
	public:
		//! enum for available resampling methods
		enum RESAMPLEMETHOD {
			RESAMPLEMETHOD_Nearest = 0,
			RESAMPLEMETHOD_Bilinear = 1,
			RESAMPLEMETHOD_CubicConv = 2
			};

		//! constructor
		SLANTGROUND();

		//! Set Surface Elevation
		void SetSurfaceElevation(double SurfaceElevation) {m_SurfaceElevation = SurfaceElevation;}
		//! Get Surface Elevation
		double GetSurfaceElevation() const {return m_SurfaceElevation;}

		//! Set Altitude
		void SetAltitude(double Altitude) {m_Altitude = Altitude;}
		//! Get Altitude
		double GetAltitude() const {return m_Altitude;}

		//! Set Slant Spacing
		void SetSlantSpacing(double SlantSpacing) {m_SlantSpacing = SlantSpacing;}
		//! Get Slant Spacing
		double GetSlantSpacing() const {return m_SlantSpacing;}

		//! Set Azimuth Spacing
		void SetAzimuthSpacing(double AzimuthSpacing) {m_AzimuthSpacing = AzimuthSpacing;}
		//! Get Azimuth Spacing
		double GetAzimuthSpacing() const {return m_AzimuthSpacing;}

		//! Set NearLook Angle
		void SetNearLookAngle(double NearLookAngle) {m_NearLookAngle = NearLookAngle;}
		//! Get NearLook Angle
		double GetNearLookAngle() const {return m_NearLookAngle;}

		//! Set FarLook Angle
		void SetFarLookAngle(double FarLookAngle) {m_FarLookAngle = FarLookAngle;}
		//! Get FarLook Angle
		double GetFarLookAngle() const {return m_FarLookAngle;}

		//! Compute output raster
		ERRVALUE Process (
			const RVC::RASTER& iraster,		//!< Input RASTER
			RVC::OBJITEM &objitem,			//!< Output OBJITEM
			RESAMPLEMETHOD resampling				//!< Resampling Method
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		double m_SurfaceElevation;
		double m_Altitude;
		double m_SlantSpacing;
		double m_AzimuthSpacing;
		double m_NearLookAngle;
		double m_FarLookAngle;
		#endif
};
}// end namespace RVC
}// end namespace OP

#endif // INC_RVC_OPSLNTGRND_H