/**
 * \file rvc/uiformrastertype.h
 * \brief RVC::UI::FORM_RASTER_TYPE class definitions
 *
 * \if NODOC
 * $Id: uiformrastertype.h_v 1.5 2008/09/02 21:41:59 mju Exp $
 *
 * $Log: uiformrastertype.h_v $
 * Revision 1.5  2008/09/02 21:41:59  mju
 * Fix doxygen woarning.
 *
 * Revision 1.4  2007/07/25 21:17:05  mju
 * Derive from form_composite.
 *
 * Revision 1.3  2007/07/09 19:31:25  mju
 * Fix getLabel return type.
 *
 * Revision 1.2  2007/07/09 19:29:26  mju
 * Add getLabel.
 *
 * Revision 1.1  2006/10/24 15:55:23  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIFORMRASTERTYPE_H
#define	INC_RVC_UIFORMRASTERTYPE_H

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_RVC_IMAGE_H
#include <rvc/image.h>
#endif

namespace RVC {
namespace UI {
	
//===================================================================================================================
//! Form containing rasterr cell types to select.
class FORM_RASTER_TYPE : public MGUI::FORM_COMPOSITE {
	public:
	
		//! Constructor.
		FORM_RASTER_TYPE (
			);

		//! Destructor.
		virtual ~FORM_RASTER_TYPE (
			);
			
		//! Create raster cell type form.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,													//!< Parent pane
			const MISTRING &label,
			RVC::IMAGE::CELLTYPE AllowedCellTypes = RVC::IMAGE::CELLTYPE_All,			//!< Allowed raster cell types to select
			RVC::IMAGE::CELLTYPE DftCellType = RVC::IMAGE::CELLTYPE_None,				//!< Default raster cell type to select
			int ListHeight = 7,																		//!< Initial list box height
			MGUI::LAYOUT_SIZEALIGN sizealign=MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Get label for alignment.
		MGUI::CTRL_LABEL& GetLabel (
			) { return (m_ComboBox.GetLabel()); }
			
		//! Get raster cell type of currently selected item.
		//! @return Cell Type selected, CELLTYPE_None if none are selected
	 	RVC::IMAGE::CELLTYPE GetSelectedCellType (
			) const;
			
		//! Set the allowed set of raster cell types to show in the list
		void SetAllowedTypes (
			RVC::IMAGE::CELLTYPE AllowedCellTypes				//!< Eg: CELLTYPE_Color, CELLTYPE_Signed_Max32, CELLTYPE_Complex...
			);

		//! Set delegate to call when user makes a selection.			
		void SetDelegateOnSelection (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set currently selected raster cell type
		void SetSelectedCellType (
			RVC::IMAGE::CELLTYPE CellType
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MGUI::FORM_COMBOBOX m_ComboBox;
		
		FORM_RASTER_TYPE (const FORM_RASTER_TYPE&);
		FORM_RASTER_TYPE& operator= (const FORM_RASTER_TYPE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//===================================================================================================================
}		// End namespace UI
}		// End namespace RVC

#endif	// INC_RVC_UIFORMRASTERTYPE_H
