/**
 * \file soa/wmsuilayerselect.h
 * \brief SOA WMS User Interface for layer selection
 *
 * \if NODOC
 * $Id: wmsuilayerselect.h_v 1.2 2007/01/25 22:12:29 mju Exp $
 *
 * $Log: wmsuilayerselect.h_v $
 * Revision 1.2  2007/01/25 22:12:29  mju
 * Add setSelection.
 *
 * Revision 1.1  2007/01/19 18:41:04  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SOA_WMSUILAYERSELECT_H
#define	INC_SOA_WMSUILAYERSELECT_H

#ifndef	INC_MGUI_FORM_H
	#include <mgui/form.h>
#endif

class MISTRINGLIST;

namespace SOA {
namespace WMS {

class CONNECTION;

//=====================================================================================================================

class UI_FORM_LAYERSELECT : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		UI_FORM_LAYERSELECT ();

		//! Destructor.
		virtual ~UI_FORM_LAYERSELECT ();

		//! Create form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Get list of selected layer names.
		const MISTRINGLIST& GetSelectedLayerNames (
			) const;

		//! Get list of selected layer styles.
		//! Will have exactly same number of list entries as returned by GetSelectedLayerNames().
		//! Layers using default still will have blank list entry.
		const MISTRINGLIST& GetSelectedLayerStyles (
			) const;

		//! Set delegate to call when layer selection changed.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set connection.
		//! If successful (and changed) will reset selected layer state to default.
		ERRVALUE SetConnection (
			const WMS::CONNECTION& Connection,
			bool notify
			);

		//! Set selected layer names and styles.
		void SetSelection (
			const MISTRINGLIST& LayerNameList,
			const MISTRINGLIST& LayerStyleList,
			bool notify
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		UI_FORM_LAYERSELECT (const UI_FORM_LAYERSELECT&);
		UI_FORM_LAYERSELECT& operator= (const UI_FORM_LAYERSELECT&);
		#endif
	};

//=====================================================================================================================

}	// End namespace WMS
}	// End namespace SOA

#endif	// INC_SOA_WMSUILAYERSELECT_H
